/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package algo;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.StringTokenizer;
import java.util.zip.GZIPInputStream;

import node.BogusNode;
import support.INetworkNode;
import support.NetworkSystem;
import support.Parameters;
import ants.AntSystem;
import ants.BalancingAnt;


public class BogusAlgorithm extends Algorithm {

	public static void main(String[] args) {
		new BogusAlgorithm(args);
	}
	
	public BogusAlgorithm(String[] args) {
		setupCLI(args);
		parseCLI();

		System.out.format("---------------------------------------------\n");
		System.out.println("Bogus Algorithm Implementation");
		System.out.println("(C) 2008 Amos Brocco <amos.brocco@unifr.ch>");
		System.out.format("---------------------------------------------\n");

		printConfiguration();
		init();
		importYaml(Parameters.INITIAL);
		System.out.format("NODES = %d\n", NetworkSystem.getNodeCount());
		System.out.format("---------------------------------------------\n");
		
		this.execute();
	}
	

	protected void addNode(String hook, String id) {
		INetworkNode node = new BogusNode(new String(id));
		if (!NetworkSystem.existNode(id))
			NetworkSystem.addNode(node);
		node.userconnect(hook);
	}


	protected void generateAnts() {
		for (String n : NetworkSystem.getNodes()) {
			if (Parameters.BALANCING) {
				if (NetworkSystem.getNode(n).getLoadManagementService()
						.getLoad() > 50)
					AntSystem.schedule(new BalancingAnt(n));
			}
		}
	}
	
	public void generateAnt(String n) {
		
	}

	private void importYaml(String path) {

		String line = null;
		String currentNode = null;
		BufferedReader br = null;

		if (path.endsWith(".gz")) {
			FileInputStream fin = null;
			try {
				fin = new FileInputStream(path);
			} catch (FileNotFoundException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		    GZIPInputStream gzis = null;
			try {
				gzis = new GZIPInputStream(fin);
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		    InputStreamReader boo = new InputStreamReader(gzis);
			br = new BufferedReader(boo);
		} else {
			try {
				br = new BufferedReader(new FileReader(path));
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}

		try {
			while ((line = br.readLine()) != null) {
				if (line.charAt(0) != ' ') {
					// Skip this
				} else {
					StringTokenizer st = new StringTokenizer(line, ":");
					String id = new String(st.nextToken().trim());
					if (id.equals("datastructure"))
						break;
					if (st.countTokens() > 0) {
						// Neighbor
						BogusNode node;
						if (!NetworkSystem.existNode(id)) {
							node = new BogusNode(new String(id));
							NetworkSystem.addNode(node);
						} else {
							node = (BogusNode) NetworkSystem.getNode(id);
						}
						NetworkSystem.getNode(currentNode).userconnect(new String(id));
					} else {
						// New node
						BogusNode node;
						if (!NetworkSystem.existNode(id)) {
							node = new BogusNode(new String(id));
							NetworkSystem.addNode(node);
						}
						currentNode = id;
					}
				}
			}
			br.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	protected void algorithmStep() {
	}

}
