/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package alpha;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class Entry {

	protected String identifier = null;
	protected Set<String> neighbors;
	protected int distance;
	protected int computed;
	protected long entryAge;
	protected long remoteTime;
	protected boolean sticky = false;
	protected boolean free = false;
	protected long nextRequestMinTime = 0;

	public Entry(String id) {
		if (id != null)
			identifier = new String(id);
		neighbors = new HashSet<String>();
		distance = -1;
		computed = -1;
		entryAge = 0;
		remoteTime = 0;
		nextRequestMinTime = 0;
	}

	public void setNextRequestMinTime(long time) {
		nextRequestMinTime = time;
	}

	public long getNextRequestMinTime() {
		return nextRequestMinTime;
	}

	public void addNeighbor(String id) {
		neighbors.add(new String(id));
	}

	public void clear() {
		neighbors.clear();
	}

	public void clearNeighbors() {
		neighbors.clear();
	}

	public void free() {
		if (sticky)
			System.out.println("Warning! Freeing sticky entry " + identifier);
		free = true;
	}

	public int getComputedDistance() {
		return computed;
	}

	public int getDistance() {
		return distance;
	}

	protected long getEntryAge() {
		return entryAge;
	}

	public String getId() {
		return identifier;
	}

	public List<String> getNeighbors() {
		List<String> ns = new ArrayList<String>(neighbors.size());
		getNeighbors(ns);
		return ns;
	}

	public void getNeighbors(List<String> dest) {
		for (String n : neighbors)
			dest.add(new String(n));
	}

	protected void getNeighbors(Set<String> dest) {
		for (String n : neighbors)
			dest.add(new String(n));
	}

	public int getNeighborsCount() {
		return neighbors.size();
	}

	public int getNeighborsHash() {
		return neighbors.hashCode();
	}

	protected long getRemoteTime() {
		return remoteTime;
	}

	public boolean isFree() {
		return free;
	}

	public boolean isNeighbor(String id) {
		return neighbors.contains(id);
	}

	public boolean isSticky() {
		return sticky;
	}

	public void removeNeighbor(String id) {
		neighbors.remove(id);
	}

	public void setComputedDistance(int d) {
		computed = d;
	}

	public boolean setDistance(int d) {
		if (distance == d) return false;
		distance = d;
		return true;
	}

	protected void setEntryAge(long a) {
		entryAge = a;
	}

	public void setId(String id) {
		identifier = new String(id);
	}

	public void setRemoteTime(long a) {
		remoteTime = a;
	}

	public void stick() {
		sticky = true;
	}

	/*public long tick() {
		remoteTime++;
		return remoteTime;
	}*/

	public String toString() {
		return String.format(" (id=%s,ns=%s,d=%d,cd=%d,age=%d,rt=%d) ",
				identifier, neighbors.toString(), distance, computed, entryAge,
				remoteTime);
	}

	public void unstick() {
		sticky = false;
	}

	public boolean updateFrom(Entry e) {
		if (e == null)
			return false;

		identifier = new String(e.getId());
		remoteTime = new Long(e.remoteTime);

		if (neighbors.hashCode() != e.getNeighborsHash()) {
			neighbors.clear();
			e.getNeighbors(neighbors);
			return true;
		}

		return false;
	}

	public boolean updateNeighbors(List<String> ns) {
		if (neighbors.size() == ns.size()) {
			for (String n : ns) {
				if (!neighbors.contains(n)) {
					clearNeighbors();
					for (String m : ns) {
						addNeighbor(new String(m));
					}
					return false;
				}
			}
		} else {
			clearNeighbors();
			for (String n : ns) {
				addNeighbor(new String(n));
			}
		}
		return true;	
	}

}
