/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package alpha;

import java.util.Iterator;

import support.IBlatAntNode;

public class Table implements Iterable<Entry> {

	private Entry[] table;
	private IBlatAntNode owner;

	public Table(IBlatAntNode owner, int size) {
		table = new Entry[size];
		this.owner = owner;
	}

	public void clear() {
		for (int i = 0; i < table.length; i++)
			table[i] = null;
	}

	private int create(int i, String e) {
		if ((table[i] == null) || (table[i].isFree()))
			table[i] = new Entry(new String(e));
		return i;
	}

	private int freeOldestIndex() {
		long timestamp = owner.getTime();
		long age;
		int index = -1;
		for (int i = 0; i < table.length; i++) {
			if (table[i] == null) {
				index = i;
				break;
			}
			// As we are called by getFreeIndex no entry is null or free
			if (table[i].isSticky())
				continue;
			age = table[i].getEntryAge();
			if (age <= timestamp) {
				timestamp = age;
				index = i;
			}
		}
		table[index].free();
		return index;
	}

	public Entry get(int i) {
		return table[i];
	}

	public Entry get(String id) {
		int z = getIndex(id);
		return z < 0 ? null : table[z];
	}

	public Entry getOrNew(String id) {
		int i = getIndexOrNew(id);
		return get(i);
	}

	public int getFreeIndex() {
		for (int i = 0; i < table.length; i++) {
			if ((table[i] == null) || (table[i].isFree()))
				return i;
		}
		//System.out.println("Table full "+table.length);
		//printTable();
		return freeOldestIndex();
	}

	public int getIndex(String id) {
		for (int i = 0; i < table.length; i++) {
			if ((table[i] != null) && (!table[i].isFree())
					&& (table[i].getId().equals(id)))
				return i;
		}
		return -1;
	}

	/*private void printTable() {
		for (int i = 0; i < table.length; i++) {
			if((table[i] != null))
				printEntry(table[i]);
		}
	}

	private void printEntry(Entry entry) {
		System.out.format("%s : %s : dist=%d : age=%d  : rtime=%d\n", entry.identifier, entry.neighbors.toString(), entry.distance, entry.entryAge, entry.remoteTime);
	}*/

	public int getIndexOrNew(String id) {
		int i = getIndex(id);
		return i == -1 ? create(getFreeIndex(), id) : i;
	}

	public Iterator<Entry> iterator() {
		return new Iterator<Entry>() {

			private int Index = 0;

			public boolean hasNext() {
				return Index < size();
			}

			public Entry next() {
				return table[Index++];
			}

			public void remove() {
				throw new UnsupportedOperationException();
			}
		};
	}

	public int size() {
		return table.length;
	}

	public void stick(int i) {
		if (i >= 0)
			table[i].stick();
	}

	public void unstick(int i) {
		if (i >= 0) {
			table[i].unstick();
		}
	}

	public void update(int i, Entry e) {
		if (table[i] == null)
			table[i] = new Entry(new String(e.getId()));
		table[i].updateFrom(e);
	}

	public boolean contains(String id) {
		return (get(id) != null) && (!get(id).isFree());
	}

}
