#!/usr/bin/python
# ./datasummary.py | gzip -c - > prova.gnumeric
import networkx as NX
import networkx.utils as NXU
import pylab as P
from math import *
from time import *
import os
from time import gmtime, strftime
import thread
import threading
import sys
import sets
import string
import pickle
import random
import gzip

def getParameter(l, parameter):
	if l.count("N=") <= 0:
		return None
	else:
		pl = l.strip().split("\t")
		iter = int(pl[0])
	if parameter == "dants":
		dants = pl[3].split("=")[1]
		return dants
	elif parameter == "olinkants":		
		linkants = pl[4].split("=")[1]
		return linkants
	elif parameter == "clinkants":			
		userlinkants = pl[5].split("=")[1]
		return userlinkants
	elif parameter == "pingants":			
		pingants = pl[6].split("=")[1]
		return pingants
	elif parameter == "unlinkants":				
		unlinkants = pl[7].split("=")[1]
		return unlinkants
	elif parameter == "nodes":				
		nodes = pl[9].split("=")[1].split("(")[0]
		return nodes
	elif parameter == "lcc":				
		largest = pl[9].split("=")[1].split("(")[1].split(")")[0]
		return largest
	elif parameter == "edges":				
		edges = pl[10].split("=")[1].split("(")[0]
		return edges
	elif parameter == "remaining":				
		remaining = pl[10].split("=")[1].split("(")[1].split(")")[0]
		return remaining
	elif parameter == "diameter":				
		diameter = pl[12].split("=")[1].split("/")[0]
		return diameter
	elif parameter == "radius":				
		radius = pl[12].split("=")[1].split("/")[1]
		return radius
	elif parameter == "avpl":				
		avpl = pl[12].split("=")[1].split("/")[2]
		return avpl
	else:
		return None	


class BogoGnumeric:
	
	def __init__(self, file):
		self.filepointer = file

	def doHeader(self):
		self.filepointer.write( '<?xml version="1.0" encoding="UTF-8"?>\n')
		self.filepointer.write('<gnm:Workbook xmlns:gnm="http://www.gnumeric.org/v10.dtd" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.gnumeric.org/v9.xsd">\n')
		self.filepointer.write('  <gnm:Version Epoch="1" Major="8" Minor="2" Full="1.8.2"/>\n')
		self.filepointer.write('  <gnm:SheetNameIndex>\n')
		self.filepointer.write('    <!-- Each sheet has its own name -->\n')
		self.filepointer.write('    <gnm:SheetName>Reliable Static</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Reliable Proper</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Reliable Improper</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Unreliable Static</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Unreliable Proper</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Unreliable Improper</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Graph</gnm:SheetName>\n')
		self.filepointer.write('  </gnm:SheetNameIndex>\n')
		self.filepointer.write('  <gnm:Geometry Width="1390" Height="807"/>\n')
		self.filepointer.write('  <gnm:Sheets>\n')
		
	def startSheet(self, name):
		self.filepointer.write('<gnm:Sheet DisplayFormulas="0" HideZero="0" HideGrid="0" HideColHeader="0" HideRowHeader="0" DisplayOutlines="1" OutlineSymbolsBelow="1" OutlineSymbolsRight="1" Visibility="GNM_SHEET_VISIBILITY_VISIBLE" GridColor="0:0:0">\n')
		self.filepointer.write('<gnm:Name>%s</gnm:Name>\n' % (name))
		self.filepointer.write('<gnm:MaxCol>127</gnm:MaxCol>\n')
		self.filepointer.write('<gnm:MaxRow>1023</gnm:MaxRow>\n')
		self.filepointer.write('<gnm:Zoom>1</gnm:Zoom>\n')
		self.filepointer.write('<gnm:Cells>\n')
		self.filepointer.write('<!-- Formulas are defined once and then just references -->\n')
		self.filepointer.write('<!-- Data is defined per row and column, starting at 0 for both -->\n')
		
	def outputFormulas(self, row):
		if (row == 1):
			self.filepointer.write('<gnm:Cell Row="1" Col="6" ExprID="1">=B2*135+C2*2+D2*2+E2+F2*2</gnm:Cell>\n')
		else:
			self.filepointer.write('<gnm:Cell Row="%d" Col="6" ExprID="1"/>\n' % (row))
		
	def outputData (self, row, col, value):
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ValueType="40">%s</gnm:Cell>\n' % (row, col, value))
		  
	def endSheet(self):
		self.filepointer.write('</gnm:Cells>\n')
		self.filepointer.write('<gnm:SheetLayout TopLeft="A1"/>\n')
		self.filepointer.write('</gnm:Sheet>\n')
	 
	def doFooter(self):
		self.filepointer.write('  </gnm:Sheets>\n')
		self.filepointer.write('  <gnm:UIData SelectedTab="0"/>\n')
		self.filepointer.write('  <gnm:Calculation ManualRecalc="0" EnableIteration="1" MaxIterations="100" IterationTolerance="0.001"/>\n')
		self.filepointer.write('</gnm:Workbook>\n')
		
	def doGraph(self, ylabel="None", ymin=-1, ymax=-1, xmin=1, xmax=1001):
		self.filepointer.write('  <gnm:Sheet DisplayFormulas="0" HideZero="0" HideGrid="0" HideColHeader="0" HideRowHeader="0" DisplayOutlines="1" OutlineSymbolsBelow="1" OutlineSymbolsRight="1" Visibility="GNM_SHEET_VISIBILITY_VISIBLE" GridColor="0:0:0">\n')
		self.filepointer.write('       <gnm:Name>Graph</gnm:Name>\n')
		self.filepointer.write('       <gnm:MaxCol>-1</gnm:MaxCol>\n')
		self.filepointer.write('       <gnm:MaxRow>-1</gnm:MaxRow>\n')
		self.filepointer.write('       <gnm:Zoom>1</gnm:Zoom>\n')
		self.filepointer.write('       <gnm:Names>\n')
		self.filepointer.write('         <gnm:Name>\n')
		self.filepointer.write('           <gnm:name>Print_Area</gnm:name>\n')
		self.filepointer.write('           <gnm:value>Graph!$A$1:$IV$65536</gnm:value>\n')
		self.filepointer.write('           <gnm:position>A1</gnm:position>\n')
		self.filepointer.write('         </gnm:Name>\n')
		self.filepointer.write('         <gnm:Name>\n')
		self.filepointer.write('           <gnm:name>Sheet_Title</gnm:name>\n')
		self.filepointer.write('           <gnm:value>&quot;Graph&quot;</gnm:value>\n')
		self.filepointer.write('           <gnm:position>A1</gnm:position>\n')
		self.filepointer.write('         </gnm:Name>\n')
		self.filepointer.write('       </gnm:Names>\n')
		self.filepointer.write('       <gnm:PrintInformation>\n')
		self.filepointer.write('         <gnm:Margins>\n')
		self.filepointer.write('           <gnm:top Points="120" PrefUnit="mm"/>\n')
		self.filepointer.write('           <gnm:bottom Points="120" PrefUnit="mm"/>\n')
		self.filepointer.write('           <gnm:left Points="72" PrefUnit="mm"/>\n')
		self.filepointer.write('           <gnm:right Points="72" PrefUnit="mm"/>\n')
		self.filepointer.write('           <gnm:header Points="72" PrefUnit="mm"/>\n')
		self.filepointer.write('           <gnm:footer Points="72" PrefUnit="mm"/>\n')
		self.filepointer.write('         </gnm:Margins>\n')
		self.filepointer.write('         <gnm:Scale type="percentage" percentage="100"/>\n')
		self.filepointer.write('         <gnm:vcenter value="0"/>\n')
		self.filepointer.write('         <gnm:hcenter value="0"/>\n')
		self.filepointer.write('         <gnm:grid value="0"/>\n')
		self.filepointer.write('         <gnm:even_if_only_styles value="0"/>\n')
		self.filepointer.write('         <gnm:monochrome value="0"/>\n')
		self.filepointer.write('         <gnm:draft value="0"/>\n')
		self.filepointer.write('         <gnm:titles value="0"/>\n')
		self.filepointer.write('         <gnm:do_not_print value="0"/>\n')
		self.filepointer.write('         <gnm:order>d_then_r</gnm:order>\n')
		self.filepointer.write('         <gnm:orientation>portrait</gnm:orientation>\n')
		self.filepointer.write('         <gnm:Header Left="" Middle="&amp;[TAB]" Right=""/>\n')
		self.filepointer.write('         <gnm:Footer Left="" Middle="Page &amp;[PAGE]" Right=""/>\n')
		self.filepointer.write('         <gnm:paper>iso_a4</gnm:paper>\n')
		self.filepointer.write('       </gnm:PrintInformation>\n')
		self.filepointer.write('       <gnm:Styles>\n')
		self.filepointer.write('         <gnm:StyleRegion startCol="0" startRow="0" endCol="255" endRow="65535">\n')
		self.filepointer.write('           <gnm:Style HAlign="1" VAlign="2" WrapText="0" ShrinkToFit="0" Rotation="0" Shade="0" Indent="0" Locked="1" Hidden="0" Fore="0:0:0" Back="FFFF:FFFF:FFFF" PatternColor="0:0:0" Format="General">\n')
		self.filepointer.write('             <gnm:Font Unit="10" Bold="0" Italic="0" Underline="0" StrikeThrough="0" Script="0">Sans</gnm:Font>\n')
		self.filepointer.write('           </gnm:Style>\n')
		self.filepointer.write('         </gnm:StyleRegion>\n')
		self.filepointer.write('       </gnm:Styles>\n')
		self.filepointer.write('       <gnm:Cols DefaultSizePts="48"/>\n')
		self.filepointer.write('       <gnm:Rows DefaultSizePts="12.75"/>\n')
		self.filepointer.write('       <gnm:Selections CursorCol="11" CursorRow="9">\n')
		self.filepointer.write('         <gnm:Selection startCol="11" startRow="9" endCol="11" endRow="9"/>\n')
		self.filepointer.write('       </gnm:Selections>\n')
		self.filepointer.write('       <gnm:Objects>\n')
		self.filepointer.write('         <gnm:SheetObjectGraph ObjectBound="A2:I21" ObjectOffset="0.938 0.588 0.438 0.412" Direction="17">\n')
		self.filepointer.write('           <GogObject type="GogGraph">\n')
		self.filepointer.write('             <property name="height-pts">240.000000</property>\n')
		self.filepointer.write('             <property name="width-pts">360.000000</property>\n')
		self.filepointer.write('             <property name="theme-name">Default</property>\n')
		self.filepointer.write('             <property name="padding-pts">7.086614</property>\n')
		self.filepointer.write('             <property name="style" type="GogStyle">\n')
		self.filepointer.write('               <outline dash="none" auto-dash="1" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('               <fill type="none" auto-type="1" is-auto="1" auto-fore="1"/>\n')
		self.filepointer.write('             </property>\n')
		self.filepointer.write('             <property name="anchor">top-left</property>\n')
		self.filepointer.write('             <property name="alignment">fill</property>\n')
		self.filepointer.write('             <GogObject role="Chart">\n')
		self.filepointer.write('               <property name="padding-pts">7.086614</property>\n')
		self.filepointer.write('               <property name="style" type="GogStyle">\n')
		self.filepointer.write('                 <outline dash="none" auto-dash="0" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                 <fill type="pattern" auto-type="1" is-auto="1" auto-fore="1">\n')
		self.filepointer.write('                   <pattern type="solid" fore="0:0:0:FF" back="FF:FF:FF:FF"/>\n')
		self.filepointer.write('                 </fill>\n')
		self.filepointer.write('               </property>\n')
		self.filepointer.write('               <property name="id">1</property>\n')
		self.filepointer.write('               <GogObject role="X-Axis">\n')
		self.filepointer.write('                 <property name="map-name">Linear</property>\n')
		self.filepointer.write('                 <property name="style" type="GogStyle">\n')
		self.filepointer.write('                   <line dash="solid" auto-dash="1" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                   <font color="0:0:0:FF" font="FreeSerif 8" auto-scale="1"/>\n')
		self.filepointer.write('                   <text_layout angle="0"/>\n')
		self.filepointer.write('                 </property>\n')
		self.filepointer.write('                 <property name="id">1</property>\n')
		self.filepointer.write('                 <data/>\n')
		self.filepointer.write('                 <GogObject role="Label">\n')
		self.filepointer.write('                   <property name="allow-markup">FALSE</property>\n')
		self.filepointer.write('                   <property name="padding-pts">7.086614</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <outline dash="none" auto-dash="1" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                     <fill type="none" auto-type="1" is-auto="1" auto-fore="1"/>\n')
		self.filepointer.write('                     <font color="0:0:0:FF" font="FreeSerif 10" auto-scale="1"/>\n')
		self.filepointer.write('                     <text_layout angle="0"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">1</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataScalar">&quot;iteration&quot;</dimension>\n')
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('               </GogObject>\n')
		self.filepointer.write('               <GogObject role="Y-Axis">\n')
		self.filepointer.write('                 <property name="map-name">Linear</property>\n')
		self.filepointer.write('                 <property name="minor-tick-out">TRUE</property>\n')
		self.filepointer.write('                 <property name="style" type="GogStyle">\n')
		self.filepointer.write('                   <line dash="solid" auto-dash="1" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                   <font color="0:0:0:FF" font="FreeSerif 8" auto-scale="1"/>\n')
		self.filepointer.write('                   <text_layout angle="0"/>\n')
		self.filepointer.write('                 </property>\n')
		self.filepointer.write('                 <property name="id">1</property>\n')
		self.filepointer.write('                 <data>\n')
		if ymin > 0:
			self.filepointer.write('                   <dimension id="0" type="GnmGODataScalar">%d</dimension>\n' % (ymin))
		if ymax > 0:
			self.filepointer.write('                   <dimension id="1" type="GnmGODataScalar">%d</dimension>\n' % (ymax))
		self.filepointer.write('                 </data>\n')
		self.filepointer.write('                 <GogObject role="Label">\n')
		self.filepointer.write('                   <property name="allow-markup">FALSE</property>\n')
		self.filepointer.write('                   <property name="padding-pts">7.086614</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <outline dash="none" auto-dash="1" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                     <fill type="none" auto-type="1" is-auto="1" auto-fore="1"/>\n')
		self.filepointer.write('                     <font color="0:0:0:FF" font="FreeSerif 10" auto-scale="1"/>\n')
		self.filepointer.write('                     <text_layout angle="90"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">1</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataScalar">&quot;%s&quot;</dimension>\n' % (ylabel))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('               </GogObject>\n')
		self.filepointer.write('               <GogObject role="Y-Axis">\n')
		self.filepointer.write('                 <property name="map-name">Linear</property>\n')
		self.filepointer.write('                 <property name="major-tick-out">FALSE</property>\n')
		self.filepointer.write('                 <property name="major-tick-labeled">FALSE</property>\n')
		self.filepointer.write('                 <property name="pos-str">cross</property>\n')
		self.filepointer.write('                 <property name="style" type="GogStyle">\n')
		self.filepointer.write('                   <line dash="dot" auto-dash="0" width="0.2" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                   <font color="0:0:0:FF" font="Sans 8" auto-scale="1"/>\n')
		self.filepointer.write('                   <text_layout angle="0"/>\n')
		self.filepointer.write('                 </property>\n')
		self.filepointer.write('                 <property name="id">2</property>\n')
		self.filepointer.write('                 <data>\n')
		self.filepointer.write('                   <dimension id="4" type="GnmGODataScalar">10</dimension>\n')
		self.filepointer.write('                 </data>\n')
		self.filepointer.write('               </GogObject>\n')
		self.filepointer.write('               <GogObject role="Y-Axis">\n')
		self.filepointer.write('                 <property name="map-name">Linear</property>\n')
		self.filepointer.write('                 <property name="major-tick-out">FALSE</property>\n')
		self.filepointer.write('                 <property name="major-tick-labeled">FALSE</property>\n')
		self.filepointer.write('                 <property name="pos-str">cross</property>\n')
		self.filepointer.write('                 <property name="style" type="GogStyle">\n')
		self.filepointer.write('                   <line dash="dot" auto-dash="0" width="0.2" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                   <font color="0:0:0:FF" font="Sans 8" auto-scale="1"/>\n')
		self.filepointer.write('                   <text_layout angle="0"/>\n')
		self.filepointer.write('                 </property>\n')
		self.filepointer.write('                 <property name="id">3</property>\n')
		self.filepointer.write('                 <data>\n')
		self.filepointer.write('                   <dimension id="4" type="GnmGODataScalar">500</dimension>\n')
		self.filepointer.write('                 </data>\n')
		self.filepointer.write('               </GogObject>\n')
		self.filepointer.write('               <GogObject role="Plot" type="GogLinePlot">\n')
		self.filepointer.write('                 <property name="default-style-has-markers">FALSE</property>\n')
		self.filepointer.write('                 <property name="y-axis">1</property>\n')
		self.filepointer.write('                 <property name="x-axis">1</property>\n')
		self.filepointer.write('                 <property name="vary-style-by-element">FALSE</property>\n')
		self.filepointer.write('                 <property name="id">1</property>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="0.3" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="0" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">1</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;1)  S-R&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Reliable Static\'!$G$%d:$G$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="0.9" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="1" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">5</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;2)  DP-R&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Reliable Proper\'!$G$%d:$G$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="1.8" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="1" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">2</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;3)  DI-R&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n'  % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Reliable Improper\'!$G$%d:$G$%d</dimension>\n'  % (xmin,xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="0.3" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="0" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">3</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;4)  S-U&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n'  % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Unreliable Static\'!$G$%d:$G$%d</dimension>\n'  % (xmin,xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="0.9" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="1" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">6</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;5)  DP-U&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Unreliable Proper\'!$G$%d:$G$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="1.8" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="0" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">4</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;6)  DI-U&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Unreliable Improper\'!$G$%d:$G$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('               </GogObject>\n')
		self.filepointer.write('               <GogObject role="Legend">\n')
		self.filepointer.write('                 <property name="swatch-padding-pts">5.669291</property>\n')
		self.filepointer.write('                 <property name="swatch-size-pts">7.086614</property>\n')
		self.filepointer.write('                 <property name="padding-pts">7.086614</property>\n')
		self.filepointer.write('                 <property name="style" type="GogStyle">\n')
		self.filepointer.write('                   <outline dash="none" auto-dash="0" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                   <fill type="pattern" auto-type="1" is-auto="1" auto-fore="1">\n')
		self.filepointer.write('                     <pattern type="solid" fore="0:0:0:FF" back="FF:FF:FF:FF"/>\n')
		self.filepointer.write('                   </fill>\n')
		self.filepointer.write('                   <font color="0:0:0:FF" font="Sans 8" auto-scale="1"/>\n')
		self.filepointer.write('                 </property>\n')
		self.filepointer.write('                 <property name="is-position-manual">TRUE</property>\n')
		self.filepointer.write('                 <property name="position">0.76754791852925619 0.012184088001831517 0.22342867004161612 0.30382617257019817</property>\n')
		self.filepointer.write('                 <property name="id">1</property>\n')
		self.filepointer.write('               </GogObject>\n')
		self.filepointer.write('             </GogObject>\n')
		self.filepointer.write('           </GogObject>\n')
		self.filepointer.write('         </gnm:SheetObjectGraph>\n')
		self.filepointer.write('       </gnm:Objects>\n')
		self.filepointer.write('       <gnm:Cells/>\n')
		self.filepointer.write('       <gnm:SheetLayout TopLeft="A1"/>\n')
		self.filepointer.write('       <gnm:Solver ProblemType="1" Inputs="" MaxTime="0" MaxIter="0" NonNeg="1" Discr="0" AutoScale="0" ShowIter="0" AnswerR="0" SensitivityR="0" LimitsR="0" PerformR="0" ProgramR="0"/>\n')
		self.filepointer.write('     </gnm:Sheet> \n')	



## FILE TEMPLATES

template_sr = "results-%d-noloss-noslowdown.log"
template_dpr = "results-%d-addleave.log"
template_dir = "results-%d-addremove.log"
template_su = "results-%d-loss-slowdown.log"
template_dpu = "results-%d-addleave-loss-slowdown.log"
template_diu = "results-%d-addrem-loss-slowdown.log"

LOGDIR = "logs/"
OUTDIR = "summary/"
PARAMETERS = ["dants", "olinkants", "clinkants", "pingants", "unlinkants"]
TEMPLATES = [template_sr, template_dpr, template_dir, template_su, template_dpu, template_diu]
TEMPLATES_TITLES = ["Reliable Static", "Reliable Proper", "Reliable Improper", "Unreliable Static", "Unreliable Proper", "Unreliable Improper"]
EXPERIMENTS = 20
STEP = 10
INTERVAL = 12


label = 0

outf = gzip.open(OUTDIR+"traffic-summary.gnumeric",'w')
b = BogoGnumeric(outf)
b.doHeader()
for k in range(0, len(TEMPLATES)): # per ogni scenario un foglio
	print "--> Processing", TEMPLATES_TITLES[k]
	b.startSheet(TEMPLATES_TITLES[k])
	column = 1
	for param in PARAMETERS: # per i diversi tipi di formica
		print "   --> Computing results for", param
		b.outputData(0, column, param)
		results = [] # lista risultati
		for i in range(1, EXPERIMENTS + 1): # for all experiments
			result = []
			# Process 
			inputfile = LOGDIR + TEMPLATES[k]
			try:
				file = open (inputfile % i, 'r')
			except:
				##print "Failed to open ",inputfile % i,"skipping"
				continue
			content = file.readlines()
			file.close()
			for line in content:
				data = getParameter(line, param)
				if data is not None:
					result.append(data)
			results.append(result) ## Aggiunge vettore risultati
		# Dal vettore results faccio nuovo vettore con la media
		max = 0
		# Ottieni il vettore massimo
		for r in results:
			if len(r) > max:
				max = len(r)
		print "   --> There are", max,"samples in this data set per experiment"
		data = []
		for i in range(0, max):
			total = 0
			samples = 0
			for r in results:
				try:
					value = r[i]
					samples += 1
				except:
					value = 0
				total += int(value)
			average = 0
			if samples > 0:
				average = total / samples
			data.append(average)
		row = 1
		step = 0
		for x in data:
			if (step % INTERVAL == 0):
				b.outputData(row, column, str(x))			
				row += 1
			step += 1				
		column += 1
	step = 0
	row = 1
	for n in range(1,1003):
		if (step % INTERVAL == 0):
			b.outputFormulas(row)
			row += 1
		step += 1
	step = 0	
	row = 1
	for iter in range(1,1003):
		if (step % INTERVAL == 0):
			b.outputData(row, 0, str(iter*STEP))	
			row += 1
		step += 1		
	b.endSheet()				

b.doGraph(ymin=6000, ymax=25000, ylabel="Tokens", xmin=2, xmax=202)
b.doFooter()
outf.close()
