#!/usr/bin/python
# ./datasummary.py | gzip -c - > prova.gnumeric
import networkx as NX
import networkx.utils as NXU
import pylab as P
from math import *
from time import *
import os
from time import gmtime, strftime
import thread
import threading
import sys
import sets
import string
import pickle
import random
import gzip

if (len(sys.argv) > 1):
	EXPERIMENTS = int(sys.argv[1])
	print "Processing",EXPERIMENTS,"experiments"
else:
	print "No experiment count given, assuming 20"
	EXPERIMENTS = 20

def getParameter(l, parameter):
	if l.count("N=") <= 0:
		return None
	else:
		pl = l.strip().split("\t")
		iter = int(pl[0])
	if parameter == "dants":
		dants = pl[3].split("=")[1]
		return dants
	elif parameter == "olinkants":		
		linkants = pl[4].split("=")[1]
		return linkants
	elif parameter == "clinkants":			
		userlinkants = pl[5].split("=")[1]
		return userlinkants
	elif parameter == "pingants":			
		pingants = pl[6].split("=")[1]
		return pingants
	elif parameter == "unlinkants":				
		unlinkants = pl[7].split("=")[1]
		return unlinkants
	elif parameter == "nodes":				
		nodes = pl[9].split("=")[1].split("(")[0]
		return nodes
	elif parameter == "lcc":				
		largest = pl[9].split("=")[1].split("(")[1].split(")")[0].split("/")[0]
		return largest
	elif parameter == "scc":				
		smallest = pl[9].split("=")[1].split("(")[1].split(")")[0].split("/")[1]
		return smallest
	elif parameter == "edges":				
		edges = pl[10].split("=")[1].split("(")[0]
		return edges
	elif parameter == "remaining":				
		remaining = pl[10].split("=")[1].split("(")[1].split(")")[0]
		return remaining
	elif parameter == "diameter":				
		diameter = pl[12].split("=")[1].split("/")[0]
		return diameter
	elif parameter == "radius":				
		radius = pl[12].split("=")[1].split("/")[1]
		return radius
	elif parameter == "avpl":				
		avpl = pl[12].split("=")[1].split("/")[2]
		return avpl
	elif parameter == "slow":
		slow = pl[13].split("=")[1].split("/")[0]
		return slow
	elif parameter == "loss":
		loss = pl[13].split("=")[1].split("/")[1]
		return loss
	elif parameter == "over":
		over = pl[14].split("=")[1]
		return over
	elif parameter == "delta":
		delta = pl[17].split("=")[1]
		return delta	
	else:
		return None	
		
def getColumnName(index):
	if (index < 26):
		return chr(ord("A")+index)
	else:
		modulo = index % 26
		return getColumnName((index/26)-1) + getColumnName(modulo) 
				
class BogoGnumeric:
	
	def __init__(self, file):
		self.filepointer = file

	def doHeader(self):
		self.filepointer.write( '<?xml version="1.0" encoding="UTF-8"?>\n')
		self.filepointer.write('<gnm:Workbook xmlns:gnm="http://www.gnumeric.org/v10.dtd" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.gnumeric.org/v9.xsd">\n')
		self.filepointer.write('  <gnm:Version Epoch="1" Major="8" Minor="2" Full="1.8.2"/>\n')
		self.filepointer.write('  <gnm:SheetNameIndex>\n')
		self.filepointer.write('    <!-- Each sheet has its own name -->\n')
		self.filepointer.write('    <gnm:SheetName>Reliable Static</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Reliable Proper</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Reliable Improper</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Unreliable Static</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Unreliable Proper</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Unreliable Improper</gnm:SheetName>\n')
		self.filepointer.write('    <gnm:SheetName>Graph</gnm:SheetName>\n')
		self.filepointer.write('  </gnm:SheetNameIndex>\n')
		self.filepointer.write('  <gnm:Geometry Width="1390" Height="807"/>\n')
		self.filepointer.write('  <gnm:Sheets>\n')
	
		
	def outputSummary(self, startrow = 1001, startcol = 20):
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ValueType="60">teta100</gnm:Cell>\n' % (startrow, startcol))
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ExprID="6">=V11</gnm:Cell>\n' % (startrow, startcol+1))
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ValueType="60">max</gnm:Cell>\n' % (startrow, startcol+2))
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ExprID="7">=max(X1:X1001)</gnm:Cell>\n' % (startrow, startcol+3))
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ValueType="60">teta5000</gnm:Cell>\n' % (startrow + 1, startcol))
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ExprID="8">=V501</gnm:Cell>\n' % (startrow + 1, startcol+1))
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ValueType="60">average</gnm:Cell>\n' % (startrow + 1, startcol+2))
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ExprID="9">=average(X1:X1001)</gnm:Cell>\n' % (startrow + 1, startcol+3))
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ValueType="60">tetaFinal</gnm:Cell>\n' % (startrow + 2, startcol))
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ExprID="10">=V1001</gnm:Cell>\n' % (startrow + 2, startcol+1))
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ValueType="60">sigmaFinal</gnm:Cell>\n' % (startrow + 2, startcol+2))
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ExprID="11">=X1001</gnm:Cell>\n' % (startrow + 2, startcol+3))


	def startSheet(self, name, maxrow = 1023):
		self.filepointer.write('<gnm:Sheet DisplayFormulas="0" HideZero="0" HideGrid="0" HideColHeader="0" HideRowHeader="0" DisplayOutlines="1" OutlineSymbolsBelow="1" OutlineSymbolsRight="1" Visibility="GNM_SHEET_VISIBILITY_VISIBLE" GridColor="0:0:0">\n')
		self.filepointer.write('<gnm:Name>%s</gnm:Name>\n' % (name))
		self.filepointer.write('<gnm:MaxCol>127</gnm:MaxCol>\n')
		self.filepointer.write('<gnm:MaxRow>%d</gnm:MaxRow>\n' % (maxrow))
		self.filepointer.write('<gnm:Zoom>1</gnm:Zoom>\n')
		self.filepointer.write('<gnm:Cells>\n')
		self.filepointer.write('<!-- Formulas are defined once and then just references -->\n')
		self.filepointer.write('<!-- Data is defined per row and column, starting at 0 for both -->\n')
		
	def outputFormulas(self, row, startcol = 21, lastcol="U"):
		if (row == 0):
			self.filepointer.write('<gnm:Cell Row="0" Col="%d" ExprID="1">=average(B1:%s1)</gnm:Cell>\n' % (startcol, lastcol))
			self.filepointer.write('<gnm:Cell Row="0" Col="%d" ExprID="2">=var(B1:%s1)</gnm:Cell>\n' % (startcol + 1, lastcol))
			self.filepointer.write('<gnm:Cell Row="0" Col="%d" ExprID="3">=stdev(B1:%s1)</gnm:Cell>\n' % (startcol + 2, lastcol))
			self.filepointer.write('<gnm:Cell Row="0" Col="%d" ExprID="4">=max(B1:%s1)</gnm:Cell>\n' % (startcol + 3, lastcol))
			self.filepointer.write(' <gnm:Cell Row="0" Col="%d" ExprID="5">=min(B1:%s1)</gnm:Cell>\n' % (startcol + 4, lastcol))
		else:
			self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ExprID="1"/>\n' % (row, startcol))
			self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ExprID="2"/>\n' % (row, startcol+1))
			self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ExprID="3"/>\n' % (row, startcol+2))
			self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ExprID="4"/>\n' % (row, startcol+3))
			self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ExprID="5"/>\n' % (row, startcol+4))
		
	def outputData (self, row, col, value):
		self.filepointer.write('<gnm:Cell Row="%d" Col="%d" ValueType="40">%s</gnm:Cell>\n' % (row, col, value))
		  
	def endSheet(self):
		self.filepointer.write('</gnm:Cells>\n')
		self.filepointer.write('<gnm:SheetLayout TopLeft="A1"/>\n')
		self.filepointer.write('</gnm:Sheet>\n')
	 
	def doFooter(self):
		self.filepointer.write('  </gnm:Sheets>\n')
		self.filepointer.write('  <gnm:UIData SelectedTab="0"/>\n')
		self.filepointer.write('  <gnm:Calculation ManualRecalc="0" EnableIteration="1" MaxIterations="100" IterationTolerance="0.001"/>\n')
		self.filepointer.write('</gnm:Workbook>\n')
		
	def addXAxis(self, id=2, crossat=500):
		self.filepointer.write('<GogObject role="X-Axis">\n')
		self.filepointer.write('	<property name="map-name">Linear</property>\n')
		self.filepointer.write('	<property name="cross-axis-id">1</property>\n')
		self.filepointer.write('	<property name="major-tick-out">FALSE</property>\n')
		self.filepointer.write('	<property name="major-tick-labeled">FALSE</property>\n')
		self.filepointer.write('	<property name="pos-str">cross</property>\n')
		self.filepointer.write('	<property name="style" type="GogStyle">\n')
		self.filepointer.write('		<line dash="dot" auto-dash="0" width="0.2" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('		<font color="0:0:0:FF" font="Sans 8" auto-scale="1"/>\n')
		self.filepointer.write('		<text_layout angle="0"/>\n')
		self.filepointer.write('	</property>\n')
		self.filepointer.write('	<property name="id">%d</property>\n' % (id))
		self.filepointer.write('	<data>\n')
		self.filepointer.write('		<dimension id="0" type="GnmGODataScalar">0</dimension>\n')
		self.filepointer.write('		<dimension id="1" type="GnmGODataScalar">20000</dimension>\n')
		self.filepointer.write('		<dimension id="2" type="GnmGODataScalar">1001</dimension>\n')
		self.filepointer.write('		<dimension id="3" type="GnmGODataScalar">1001</dimension>\n')
		self.filepointer.write('		<dimension id="4" type="GnmGODataScalar">%d</dimension>\n' % (crossat))
		self.filepointer.write('	</data>\n')
		self.filepointer.write('	</GogObject>\n')
		
	def addYAxis(self, id=3, crossat=10):		
		self.filepointer.write('               <GogObject role="Y-Axis">\n')
		self.filepointer.write('                 <property name="map-name">Linear</property>\n')
		self.filepointer.write('                 <property name="major-tick-out">FALSE</property>\n')
		self.filepointer.write('                 <property name="major-tick-labeled">FALSE</property>\n')
		self.filepointer.write('                 <property name="pos-str">cross</property>\n')
		self.filepointer.write('                 <property name="style" type="GogStyle">\n')
		self.filepointer.write('                   <line dash="dot" auto-dash="0" width="0.2" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                   <font color="0:0:0:FF" font="Sans 8" auto-scale="1"/>\n')
		self.filepointer.write('                   <text_layout angle="0"/>\n')
		self.filepointer.write('                 </property>\n')
		self.filepointer.write('                 <property name="id">%d</property>\n' % (id))
		self.filepointer.write('                 <data>\n')
		self.filepointer.write('                   <dimension id="4" type="GnmGODataScalar">%d</dimension>\n' % (crossat))
		self.filepointer.write('                 </data>\n')
		self.filepointer.write('               </GogObject>\n')		
		
	def doGraphHead(self, ylabel="None", ymin=-1, ymax=-1, xmin=1, xmax=1001, plotcol="V"):
		self.filepointer.write('  <gnm:Sheet DisplayFormulas="0" HideZero="0" HideGrid="0" HideColHeader="0" HideRowHeader="0" DisplayOutlines="1" OutlineSymbolsBelow="1" OutlineSymbolsRight="1" Visibility="GNM_SHEET_VISIBILITY_VISIBLE" GridColor="0:0:0">\n')
		self.filepointer.write('       <gnm:Name>Graph</gnm:Name>\n')
		self.filepointer.write('       <gnm:MaxCol>-1</gnm:MaxCol>\n')
		self.filepointer.write('       <gnm:MaxRow>-1</gnm:MaxRow>\n')
		self.filepointer.write('       <gnm:Zoom>1</gnm:Zoom>\n')
		self.filepointer.write('       <gnm:Names>\n')
		self.filepointer.write('         <gnm:Name>\n')
		self.filepointer.write('           <gnm:name>Print_Area</gnm:name>\n')
		self.filepointer.write('           <gnm:value>Graph!$A$1:$IV$65536</gnm:value>\n')
		self.filepointer.write('           <gnm:position>A1</gnm:position>\n')
		self.filepointer.write('         </gnm:Name>\n')
		self.filepointer.write('         <gnm:Name>\n')
		self.filepointer.write('           <gnm:name>Sheet_Title</gnm:name>\n')
		self.filepointer.write('           <gnm:value>&quot;Graph&quot;</gnm:value>\n')
		self.filepointer.write('           <gnm:position>A1</gnm:position>\n')
		self.filepointer.write('         </gnm:Name>\n')
		self.filepointer.write('       </gnm:Names>\n')
		self.filepointer.write('       <gnm:PrintInformation>\n')
		self.filepointer.write('         <gnm:Margins>\n')
		self.filepointer.write('           <gnm:top Points="120" PrefUnit="mm"/>\n')
		self.filepointer.write('           <gnm:bottom Points="120" PrefUnit="mm"/>\n')
		self.filepointer.write('           <gnm:left Points="72" PrefUnit="mm"/>\n')
		self.filepointer.write('           <gnm:right Points="72" PrefUnit="mm"/>\n')
		self.filepointer.write('           <gnm:header Points="72" PrefUnit="mm"/>\n')
		self.filepointer.write('           <gnm:footer Points="72" PrefUnit="mm"/>\n')
		self.filepointer.write('         </gnm:Margins>\n')
		self.filepointer.write('         <gnm:Scale type="percentage" percentage="100"/>\n')
		self.filepointer.write('         <gnm:vcenter value="0"/>\n')
		self.filepointer.write('         <gnm:hcenter value="0"/>\n')
		self.filepointer.write('         <gnm:grid value="0"/>\n')
		self.filepointer.write('         <gnm:even_if_only_styles value="0"/>\n')
		self.filepointer.write('         <gnm:monochrome value="0"/>\n')
		self.filepointer.write('         <gnm:draft value="0"/>\n')
		self.filepointer.write('         <gnm:titles value="0"/>\n')
		self.filepointer.write('         <gnm:do_not_print value="0"/>\n')
		self.filepointer.write('         <gnm:order>d_then_r</gnm:order>\n')
		self.filepointer.write('         <gnm:orientation>portrait</gnm:orientation>\n')
		self.filepointer.write('         <gnm:Header Left="" Middle="&amp;[TAB]" Right=""/>\n')
		self.filepointer.write('         <gnm:Footer Left="" Middle="Page &amp;[PAGE]" Right=""/>\n')
		self.filepointer.write('         <gnm:paper>iso_a4</gnm:paper>\n')
		self.filepointer.write('       </gnm:PrintInformation>\n')
		self.filepointer.write('       <gnm:Styles>\n')
		self.filepointer.write('         <gnm:StyleRegion startCol="0" startRow="0" endCol="255" endRow="65535">\n')
		self.filepointer.write('           <gnm:Style HAlign="1" VAlign="2" WrapText="0" ShrinkToFit="0" Rotation="0" Shade="0" Indent="0" Locked="1" Hidden="0" Fore="0:0:0" Back="FFFF:FFFF:FFFF" PatternColor="0:0:0" Format="General">\n')
		self.filepointer.write('             <gnm:Font Unit="10" Bold="0" Italic="0" Underline="0" StrikeThrough="0" Script="0">Sans</gnm:Font>\n')
		self.filepointer.write('           </gnm:Style>\n')
		self.filepointer.write('         </gnm:StyleRegion>\n')
		self.filepointer.write('       </gnm:Styles>\n')
		self.filepointer.write('       <gnm:Cols DefaultSizePts="48"/>\n')
		self.filepointer.write('       <gnm:Rows DefaultSizePts="12.75"/>\n')
		self.filepointer.write('       <gnm:Selections CursorCol="11" CursorRow="9">\n')
		self.filepointer.write('         <gnm:Selection startCol="11" startRow="9" endCol="11" endRow="9"/>\n')
		self.filepointer.write('       </gnm:Selections>\n')
		self.filepointer.write('       <gnm:Objects>\n')
		self.filepointer.write('         <gnm:SheetObjectGraph ObjectBound="A2:I21" ObjectOffset="0.938 0.588 0.438 0.412" Direction="17">\n')
		self.filepointer.write('           <GogObject type="GogGraph">\n')
		self.filepointer.write('             <property name="height-pts">240.000000</property>\n')
		self.filepointer.write('             <property name="width-pts">360.000000</property>\n')
		self.filepointer.write('             <property name="theme-name">Default</property>\n')
		self.filepointer.write('             <property name="padding-pts">7.086614</property>\n')
		self.filepointer.write('             <property name="style" type="GogStyle">\n')
		self.filepointer.write('               <outline dash="none" auto-dash="1" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('               <fill type="none" auto-type="1" is-auto="1" auto-fore="1"/>\n')
		self.filepointer.write('             </property>\n')
		self.filepointer.write('             <property name="anchor">top-left</property>\n')
		self.filepointer.write('             <property name="alignment">fill</property>\n')
		self.filepointer.write('             <GogObject role="Chart">\n')
		self.filepointer.write('               <property name="padding-pts">7.086614</property>\n')
		self.filepointer.write('               <property name="style" type="GogStyle">\n')
		self.filepointer.write('                 <outline dash="none" auto-dash="0" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                 <fill type="pattern" auto-type="1" is-auto="1" auto-fore="1">\n')
		self.filepointer.write('                   <pattern type="solid" fore="0:0:0:FF" back="FF:FF:FF:FF"/>\n')
		self.filepointer.write('                 </fill>\n')
		self.filepointer.write('               </property>\n')
		self.filepointer.write('               <property name="id">1</property>\n')
		self.filepointer.write('               <GogObject role="X-Axis">\n')
		self.filepointer.write('                 <property name="map-name">Linear</property>\n')
		self.filepointer.write('                 <property name="style" type="GogStyle">\n')
		self.filepointer.write('                   <line dash="solid" auto-dash="1" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                   <font color="0:0:0:FF" font="FreeSerif 8" auto-scale="1"/>\n')
		self.filepointer.write('                   <text_layout angle="0"/>\n')
		self.filepointer.write('                 </property>\n')
		self.filepointer.write('                 <property name="id">1</property>\n')
		self.filepointer.write('                 <data/>\n')
		self.filepointer.write('                 <GogObject role="Label">\n')
		self.filepointer.write('                   <property name="allow-markup">FALSE</property>\n')
		self.filepointer.write('                   <property name="padding-pts">7.086614</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <outline dash="none" auto-dash="1" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                     <fill type="none" auto-type="1" is-auto="1" auto-fore="1"/>\n')
		self.filepointer.write('                     <font color="0:0:0:FF" font="FreeSerif 10" auto-scale="1"/>\n')
		self.filepointer.write('                     <text_layout angle="0"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">1</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataScalar">&quot;iteration&quot;</dimension>\n')
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('               </GogObject>\n')
		self.filepointer.write('               <GogObject role="Y-Axis">\n')
		self.filepointer.write('                 <property name="map-name">Linear</property>\n')
		self.filepointer.write('                 <property name="minor-tick-out">TRUE</property>\n')
		self.filepointer.write('                 <property name="style" type="GogStyle">\n')
		self.filepointer.write('                   <line dash="solid" auto-dash="1" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                   <font color="0:0:0:FF" font="FreeSerif 8" auto-scale="1"/>\n')
		self.filepointer.write('                   <text_layout angle="0"/>\n')
		self.filepointer.write('                 </property>\n')
		self.filepointer.write('                 <property name="id">1</property>\n')
		self.filepointer.write('                 <data>\n')
		if ymin > 0:
			self.filepointer.write('                   <dimension id="0" type="GnmGODataScalar">%d</dimension>\n' % (ymin))
		if ymax > 0:
			self.filepointer.write('                   <dimension id="1" type="GnmGODataScalar">%d</dimension>\n' % (ymax))
		self.filepointer.write('                 </data>\n')
		self.filepointer.write('                 <GogObject role="Label">\n')
		self.filepointer.write('                   <property name="allow-markup">FALSE</property>\n')
		self.filepointer.write('                   <property name="padding-pts">7.086614</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <outline dash="none" auto-dash="1" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                     <fill type="none" auto-type="1" is-auto="1" auto-fore="1"/>\n')
		self.filepointer.write('                     <font color="0:0:0:FF" font="FreeSerif 10" auto-scale="1"/>\n')
		self.filepointer.write('                     <text_layout angle="90"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">1</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataScalar">&quot;%s&quot;</dimension>\n' % (ylabel))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('               </GogObject>\n')
		self.filepointer.write('               <GogObject role="Y-Axis">\n')
		self.filepointer.write('                 <property name="map-name">Linear</property>\n')
		self.filepointer.write('                 <property name="major-tick-out">FALSE</property>\n')
		self.filepointer.write('                 <property name="major-tick-labeled">FALSE</property>\n')
		self.filepointer.write('                 <property name="pos-str">cross</property>\n')
		self.filepointer.write('                 <property name="style" type="GogStyle">\n')
		self.filepointer.write('                   <line dash="dot" auto-dash="0" width="0.2" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                   <font color="0:0:0:FF" font="Sans 8" auto-scale="1"/>\n')
		self.filepointer.write('                   <text_layout angle="0"/>\n')
		self.filepointer.write('                 </property>\n')
		self.filepointer.write('                 <property name="id">2</property>\n')
		self.filepointer.write('                 <data>\n')
		self.filepointer.write('                   <dimension id="4" type="GnmGODataScalar">10</dimension>\n')
		self.filepointer.write('                 </data>\n')
		self.filepointer.write('               </GogObject>\n')
		self.filepointer.write('               <GogObject role="Y-Axis">\n')
		self.filepointer.write('                 <property name="map-name">Linear</property>\n')
		self.filepointer.write('                 <property name="major-tick-out">FALSE</property>\n')
		self.filepointer.write('                 <property name="major-tick-labeled">FALSE</property>\n')
		self.filepointer.write('                 <property name="pos-str">cross</property>\n')
		self.filepointer.write('                 <property name="style" type="GogStyle">\n')
		self.filepointer.write('                   <line dash="dot" auto-dash="0" width="0.2" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                   <font color="0:0:0:FF" font="Sans 8" auto-scale="1"/>\n')
		self.filepointer.write('                   <text_layout angle="0"/>\n')
		self.filepointer.write('                 </property>\n')
		self.filepointer.write('                 <property name="id">3</property>\n')
		self.filepointer.write('                 <data>\n')
		self.filepointer.write('                   <dimension id="4" type="GnmGODataScalar">500</dimension>\n')
		self.filepointer.write('                 </data>\n')
		self.filepointer.write('               </GogObject>\n')
		
	def doGraphTail(self, ylabel="None", ymin=-1, ymax=-1, xmin=1, xmax=1001, plotcol="V"):		
		self.filepointer.write('               <GogObject role="Plot" type="GogLinePlot">\n')
		self.filepointer.write('                 <property name="default-style-has-markers">FALSE</property>\n')
		self.filepointer.write('                 <property name="y-axis">1</property>\n')
		self.filepointer.write('                 <property name="x-axis">1</property>\n')
		self.filepointer.write('                 <property name="vary-style-by-element">FALSE</property>\n')
		self.filepointer.write('                 <property name="id">1</property>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="0.3" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="0" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">1</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;1)  S-R&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Reliable Static\'!$%s$%d:$%s$%d</dimension>\n' % (plotcol, xmin, plotcol, xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="0.9" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="1" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">5</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;2)  DP-R&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Reliable Proper\'!$%s$%d:$%s$%d</dimension>\n' % (plotcol, xmin, plotcol, xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="1.8" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="1" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">2</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;3)  DI-R&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n'  % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Reliable Improper\'!$%s$%d:$%s$%d</dimension>\n'  % (plotcol, xmin, plotcol, xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="0.3" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="0" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">3</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;4)  S-U&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n'  % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Unreliable Static\'!$%s$%d:$%s$%d</dimension>\n'  % (plotcol, xmin, plotcol, xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="0.9" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="1" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">6</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;5)  DP-U&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Unreliable Proper\'!$%s$%d:$%s$%d</dimension>\n' % (plotcol, xmin, plotcol, xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('                 <GogObject role="Series">\n')
		self.filepointer.write('                   <property name="fill-type">y-origin</property>\n')
		self.filepointer.write('                   <property name="style" type="GogStyle">\n')
		self.filepointer.write('                     <line dash="solid" auto-dash="0" width="1.8" color="0:0:0:FF" auto-color="0"/>\n')
		self.filepointer.write('                     <marker auto-shape="0" shape="none" auto-outline="1" outline-color="0:0:0:FF" auto-fill="1" fill-color="FF:FF:FF:FF" size="5"/>\n')
		self.filepointer.write('                   </property>\n')
		self.filepointer.write('                   <property name="id">4</property>\n')
		self.filepointer.write('                   <data>\n')
		self.filepointer.write('                     <dimension id="-1" type="GnmGODataScalar">&quot;6)  DI-U&quot;</dimension>\n')
		self.filepointer.write('                     <dimension id="0" type="GnmGODataVector">\'Reliable Static\'!$A$%d:$A$%d</dimension>\n' % (xmin,xmax))
		self.filepointer.write('                     <dimension id="1" type="GnmGODataVector">\'Unreliable Improper\'!$%s$%d:$%s$%d</dimension>\n' % (plotcol, xmin, plotcol, xmax))
		self.filepointer.write('                   </data>\n')
		self.filepointer.write('                 </GogObject>\n')
		self.filepointer.write('               </GogObject>\n')
		self.filepointer.write('               <GogObject role="Legend">\n')
		self.filepointer.write('                 <property name="swatch-padding-pts">5.669291</property>\n')
		self.filepointer.write('                 <property name="swatch-size-pts">7.086614</property>\n')
		self.filepointer.write('                 <property name="padding-pts">7.086614</property>\n')
		self.filepointer.write('                 <property name="style" type="GogStyle">\n')
		self.filepointer.write('                   <outline dash="none" auto-dash="0" width="0" color="0:0:0:FF" auto-color="1"/>\n')
		self.filepointer.write('                   <fill type="pattern" auto-type="1" is-auto="1" auto-fore="1">\n')
		self.filepointer.write('                     <pattern type="solid" fore="0:0:0:FF" back="FF:FF:FF:FF"/>\n')
		self.filepointer.write('                   </fill>\n')
		self.filepointer.write('                   <font color="0:0:0:FF" font="Sans 8" auto-scale="1"/>\n')
		self.filepointer.write('                 </property>\n')
		self.filepointer.write('                 <property name="is-position-manual">TRUE</property>\n')
		self.filepointer.write('                 <property name="position">0.76754791852925619 0.012184088001831517 0.22342867004161612 0.30382617257019817</property>\n')
		self.filepointer.write('                 <property name="id">1</property>\n')
		self.filepointer.write('               </GogObject>\n')
		self.filepointer.write('             </GogObject>\n')
		self.filepointer.write('           </GogObject>\n')
		self.filepointer.write('         </gnm:SheetObjectGraph>\n')
		self.filepointer.write('       </gnm:Objects>\n')
		self.filepointer.write('       <gnm:Cells/>\n')
		self.filepointer.write('       <gnm:SheetLayout TopLeft="A1"/>\n')
		self.filepointer.write('       <gnm:Solver ProblemType="1" Inputs="" MaxTime="0" MaxIter="0" NonNeg="1" Discr="0" AutoScale="0" ShowIter="0" AnswerR="0" SensitivityR="0" LimitsR="0" PerformR="0" ProgramR="0"/>\n')
		self.filepointer.write('     </gnm:Sheet> \n')	



## FILE TEMPLATES

template_sr = "results-%d-noloss-noslowdown.log"
template_dpr = "results-%d-addleave.log"
template_dir = "results-%d-addremove.log"
template_su = "results-%d-loss-slowdown.log"
template_dpu = "results-%d-addleave-loss-slowdown.log"
template_diu = "results-%d-addrem-loss-slowdown.log"

LOGDIR = "logs/"
OUTDIR = "summary/"
PARAMETERS = ["dants", "olinkants", "clinkants", "pingants", "unlinkants", "nodes", "lcc", "scc", "edges", "remaining", "diameter", "radius", "avpl", "slow", "loss", "over"]
LABELS = ["discovery ants", "optimization link ants", "construction link ants", "ping ants", "unlink ants", "nodes", "largest connected component", "smallest connected component", "edges", "free edges", "diameter", "radius", "average path length", "slowed down packets", "lost packets", "overloaded nodes"]
TEMPLATES = [template_sr, template_dpr, template_dir, template_su, template_dpu, template_diu]
TEMPLATES_TITLES = ["Reliable Static", "Reliable Proper", "Reliable Improper", "Unreliable Static", "Unreliable Proper", "Unreliable Improper"]
##EXPERIMENTS = 20
STEP = 10
##ITERS = 1000
ITERS = 1

label = 0
for param in PARAMETERS:
	outf = gzip.open(OUTDIR+"summary-%s.gnumeric" % (param),'w')
	print "Generating file ", OUTDIR+"summary-%s.gnumeric" % (param), "for parameter", param,"..."
	b = BogoGnumeric(outf)
	b.doHeader()
	for k in range(0, len(TEMPLATES)):
		print "   ---> ", TEMPLATES_TITLES[k], ":"
		b.startSheet(TEMPLATES_TITLES[k])
		for i in range(1, EXPERIMENTS + 1): # This gives the column
			# Process 
			inputfile = LOGDIR + TEMPLATES[k]
			try:
				file = open (inputfile % i, 'r')
			except:
				print "Failed to open ",inputfile % i,"skipping"
				continue
			print "       --->", inputfile % i
			content = file.readlines()
			file.close()
			iteration = 0
			for line in content:
				try:
					data = getParameter(line, param)
				except:
					#print "Failed to get parameter", param,"at line",line
					data = None
				if data is not None:
					b.outputData(iteration, i, str(data))
					iteration += 1
			ITERS = max(ITERS, iteration) - 1
		print "Found", ITERS, "iterations"
		for iter in range(0, ITERS + 2):
			b.outputData(iter, 0, str(iter*STEP))		
		for n in range(0, ITERS + 2):
			b.outputFormulas(n, startcol=EXPERIMENTS+1, lastcol=getColumnName(EXPERIMENTS))
		b.outputSummary(startrow=ITERS+2)
		b.endSheet()				
	b.doGraphHead(ylabel=LABELS[label], xmax=ITERS+2, plotcol=getColumnName(EXPERIMENTS+1))
	if param == "edges":
		b.addXAxis(id=2, crossat=7686)
		b.addXAxis(id=3, crossat=10248)
	if param == "avpl":
		b.addXAxis(id=2, crossat=5)
		b.addXAxis(id=3, crossat=9)
	if param == "diameter":
		b.addXAxis(id=2, crossat=5)
		b.addXAxis(id=3, crossat=9)				
	if ITERS > 1000:
		b.addYAxis(id=4, crossat=1000)
	b.doGraphTail(ylabel=LABELS[label], xmax=ITERS+2, plotcol=getColumnName(EXPERIMENTS+1))		
	b.doFooter()
	outf.close()
	label += 1
