/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package ants;

import support.IBlatAntNode;
import support.INetworkNode;
import support.NetworkSystem;
import support.Parameters;
import support.Randomizer;
import support.Stats;
import alpha.Entry;

public abstract class Ant {

	protected int age = 0;
	protected Entry lastInfo;
	protected String current_;
	protected String source;
	protected String previous_;
	protected long scheduleAfter = -1;

	protected transient INetworkNode current;
	protected transient NetworkSystem ns;

	// Should only dant increase the timestamp?

	public Ant(String current) {
		source = current;
		Stats.antBirth();
		this.current_ = current;
		this.current = NetworkSystem.getNode(current_);
		lastInfo = new Entry(current);
		previous_ = null;
	}

	// TODO: Check if this actually works
	protected void aging() {
		age++;
		if (age >= Parameters.ANT_MAX_AGE) {
			AntSystem.unschedule(this);
		}
	}

	public abstract void antstep();

	public int getAge() {
		return age;
	}

	public void kill() {
		AntSystem.unschedule(this);
	}

	protected boolean migrate(String target) {
		if (current == null) {
			// We are on a dead node
			kill();
			return false;
		}
		
		if (Parameters.PACKET_LOSS_PROB != 0 && Randomizer.pop() < Parameters.PACKET_LOSS_PROB) {
			Stats.loss();
			kill();
			return false;
		} 

		if (NetworkSystem.isNodeAlive(target)) {
			setScheduleAfter(NetworkSystem.getRTT(current_, target));
			if (Parameters.PACKET_SLOW_PROB != 0 && Randomizer.pop() < Parameters.PACKET_SLOW_PROB) {
				Stats.slow();
				setScheduleAfter(getScheduleAfter() + Parameters.MS_PER_ITER);
			}
			current.acquire();
			if (current.getGamma() != null)
				current.getGamma().fill(target);
			current.release();

			statmigration();
			current = NetworkSystem.getNode(target);
			current.acquire();
			if (current.getBeta() != null)
				current.getBeta().fill(new String(current_));
			previous_ = current_;
			if (current instanceof IBlatAntNode) {
				((IBlatAntNode) current).activateNeighbor(previous_); // Activate
																		// the
																		// neighbor,
																		// if
																		// necessary
			}
			current_ = new String(target);
			current.release();

			return true;
		} else {
			kill();
			return false;
		}
	}

	public void hook(NetworkSystem n, String current) {

	}

	public void reset(String current) {
		source = current;
		this.current_ = current;
		this.current = NetworkSystem.getNode(current_);
		lastInfo = new Entry(current);
		age = 0;
		previous_ = null;
	}

	protected void save() {
		if (current instanceof IBlatAntNode) {
			((IBlatAntNode) current).getInfosTo(lastInfo);
		}
	}

	protected void tell() {
		if (current instanceof IBlatAntNode) {
			((IBlatAntNode) current).updateEntry(lastInfo, 1);
		}
	}

	public abstract void statmigration();
	
	protected void setScheduleAfter(long value) {
		if (Parameters.RTT_MIGRATION)
			scheduleAfter = NetworkSystem.getGlobalTimeIter() + value;
	}
	
	public long getScheduleAfter() {
		return scheduleAfter;
	}
	
	public String toString() {
		return String.format("%d",scheduleAfter);
	}

}
