/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package ants;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import support.IBlatAntNode;
import support.NetworkSystem;
import support.Stats;

public class ConstructionLinkAnt extends Ant {

	private String target = null;
	private int step = 0;
	private int ttl = 0;
	private String avoid = null;

	public ConstructionLinkAnt(String id, String target, int ttl) {
		super(id);
		this.target = target;
		this.ttl = ttl;
	}

	public ConstructionLinkAnt(String id, String target, String avoid, int ttl) {
		super(id);
		this.target = target;
		this.ttl = ttl;
		this.avoid = avoid;
	}

	public void antstep() {

		if (!NetworkSystem.isNodeAlive(current_)) {
			// We are on a dead node
			kill();
			return;
		}
		if (step == 0) {
			// 1. Migrate to target
			if (migrate(target))
				step = 1;
			return;
		} else if (step == 1) {
			current.acquire();
			if (((IBlatAntNode) current).userconnect(source, ttl)) {
				// Connection request accepted
				target = new String(current_);
				current.release();
				if (migrate(source))
					step = 2;
				else {
					kill();
					return;
				}
				return;
			} else {
				Stats.notifyForwarding();
				// Connection request not accepted, but forwarded to neighbor
				// with least
				// number of neighbors

				// Sort nodes according to their known degree, we
				// prefer to connect to nodes with low degree
				List<String> ns = new ArrayList<String>(current.getNeighbors());
				// Remove the source
				ns.remove(source);
				// Remove the previous node
				ns.remove(previous_);
				// Find the neighbor with the lowest size
				Collections.sort(ns, new Comparator<String>() {
					public int compare(String arg0, String arg1) {
						return ((Integer) current
								.getNeighborNeighborsSize(arg0))
								.compareTo((Integer) current
										.getNeighborNeighborsSize(arg1));
					}

				});

				List<String> tbr = new ArrayList<String>();
				for (String n : ns) {
					if (!((IBlatAntNode) current).isActiveNeighbor(n))
						tbr.add(new String(n));
				}
				ns.removeAll(tbr);

				current.release();

				if (ns.isEmpty()) {
					kill();
					return;
				}

				for (String id : ns) {
					if (id.equals(avoid))
						continue;
					try {
						if (!id.equals(source) && !(id.equals(previous_))) {
							migrate(new String(id));
							ttl--;
							break;
						}
					} catch (Exception e) {

						// FIXME: Here we probably loop,
						// hence the ant never dies
						e.printStackTrace();
					}
				}
				return;
			}
		} else if (step == 2) {
			// Back to the source node, do a normal connection
			current.acquire();
			if (current.userconnect(target)) {

			}
			current.release();
			kill();
			step = 77;
			return;
		}
	}

	@Override
	public void statmigration() {
		Stats.antMigration(ConstructionLinkAnt.class, 1);
	}

}
