/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package ants;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import services.dss.DSSStats;
import services.dss.profile.ResourceProfile;
import support.IBlatAntNode;
import support.NetworkSystem;
import support.Parameters;
import support.Randomizer;

public class DSSRequestAnt extends Ant {

	private ResourceProfile profile;
	private String sourcetabu;
	private String previoustabu;
	private int ttl;

	public DSSRequestAnt(String source, String previous, String target, ResourceProfile query, int ttl) {
		super(target);
		this.profile = query;
		this.ttl = ttl;
		this.sourcetabu = source;
		this.previoustabu = previous;
		setScheduleAfter(NetworkSystem.getRTT(previous, target));
	}

	@Override
	public void antstep() {
		if ((current == null) || !NetworkSystem.isNodeAlive(current_)) {
			// We are on a dead node
			kill();
			return;
		}
		
		current.acquire();
		
		// Manage beta pheromone
		if (this.previoustabu != null) {
			if (current instanceof IBlatAntNode) {
				((IBlatAntNode) current).activateNeighbor(previoustabu);
			}
			if (current.getBeta() != null) {
				current.getBeta().fill(new String(previoustabu));
			}
		}
		
		// Check the query
		
		if (!current.getDSSService().notifyQuery(profile.id)) {
			current.release();
			kill();
			return;
		}
			
		if (current.getDSSService().request(profile, sourcetabu)) {
			if (!current.getIdentifier().equals(source)) {
				current.release();
				kill();
				return;
			}
		}
		//System.out.println("Request ant from "+current_+" ttl="+ttl+" ffw="+Parameters.SEARCH_LIMIT_FFW);
		// Forward the query
		
		if (ttl > 0) {
			--ttl; // Count one hop
			List<String> candidates = new ArrayList<String>();
			candidates.addAll(current.getNeighbors());
			Collections.shuffle(candidates, Randomizer.getRandom());
			int limitffw = Parameters.SEARCH_LIMIT_FFW;
			int count = 0;
		
			for (String q : candidates) {
				if ((limitffw != -1) && (count >= limitffw))
					break;
				if ((!q.equals(sourcetabu)) && (!q.equals(current_)) && (!q.equals(previoustabu))) {
					AntSystem.schedule(new DSSRequestAnt(new String(sourcetabu), new String(current_), new String(q), profile, ttl));
					//System.out.println("Request ant from "+current_+" to "+q);
					//Stats.queryHop(profile.toString(), current.getIdentifier(), q);
					DSSStats.requestHop();
					statmigration();
					if (current.getGamma() != null)
						current.getGamma().fill(q);
					count++;
				}
			}			
		}
		
		current.release();
		kill();
		return;
	}

	@Override
	public void statmigration() {
	}

}
