/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package ants;

import support.IBlatAntNode;
import support.NetworkSystem;
import support.Parameters;
import support.Stats;

public class OptimizationLinkAnt extends Ant {

	private String target = null;
	private int step = 0;
	private int Z = Parameters.D * 2 - Parameters.K;

	public OptimizationLinkAnt(String id, String target) {
		super(id);
		this.target = target;
	}

	public void antstep() {
		if (!NetworkSystem.isNodeAlive(current_)) {
			// We are on a dead node
			kill();
			return;
		}
		if (step == 0) {
			// 1. Migrate to target
			if (migrate(target))
				step = 1;
			return;
		} else if (step == 1) {
			current.acquire();
			if ((((IBlatAntNode) current).getEstimatedDistance(source) < 0)
					|| (((IBlatAntNode) current).getEstimatedDistance(source) >= Z)) {
				if (current.connect(source)) {
					// Connection request accepted
					target = new String(current_);
					current.release();
					if (migrate(source))
						step = 2;
					return;
				} else {
					current.release();
					step = 77;
					kill();
					return;
				}
			} else {
				current.release();
				step = 77;
				kill();
				return;
			}
		} else if (step == 2) {
			// Back to the source node, do a normal connection
			current.acquire();
			if (current.connect(target))
				AntSystem.schedule(new PingAnt(new String(current_),
						new String(target)));

			current.release();
			kill();
			step = 77;
			return;
		}
	}

	@Override
	public void statmigration() {
		Stats.antMigration(OptimizationLinkAnt.class, 0);
	}

}
