/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package ants;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import services.dss.profile.ResourceProfile;
import support.IBlatAntNode;
import support.NetworkSystem;
import support.Parameters;
import support.Randomizer;
import support.Stats;

public class ReplicantAnt extends Ant {

	private ResourceProfile profile;
	private String sourcetabu;
	private String previoustabu;
	private int ttl;

	public ReplicantAnt(String source, String previous, String target, ResourceProfile p, int ttl) {
		super(target);
		this.ttl = ttl;
		this.profile = p;
		this.sourcetabu = source;
		this.previoustabu = previous;
		setScheduleAfter(NetworkSystem.getRTT(previous, target));
	}

	@Override
	public void antstep() {
		
		if ((current == null) || !NetworkSystem.isNodeAlive(current_)) {
			// We are on a dead node
			kill();
			return;
		}
		
		current.acquire();
		
		// Manage beta pheromone
		if (this.previoustabu != null) {
			if (current instanceof IBlatAntNode) {
				((IBlatAntNode) current).activateNeighbor(previoustabu);
			}
			if (current.getBeta() != null) {
				current.getBeta().fill(new String(previoustabu));
			}
		}
			
		// Forward the query
		
		if (ttl > 0) {
			--ttl; // Count one hop
			List<String> candidates = new ArrayList<String>();
			candidates.addAll(current.getNeighbors());
			Collections.shuffle(candidates, Randomizer.getRandom());
			int count = 0;
			for (String q : candidates) {
				if ((!q.equals(sourcetabu)) && (!q.equals(current_)) && (!q.equals(previoustabu))) {
					AntSystem.schedule(new ReplicantAnt(new String(sourcetabu), new String(current_), new String(q), profile, ttl));
					Stats.replicantHop();
					statmigration();
					if (current.getGamma() != null)
						current.getGamma().fill(q);
					count++;
					if (!Parameters.ONE_HOP_REPLICATION) break; // Only one neighbor
				}
			}
			if ((count == 0) && (!current_.equals(sourcetabu))) {
				current.getResourceManagementService().publish(sourcetabu, profile);
			}
		} else {
			current.getResourceManagementService().publish(sourcetabu, profile);
		}
		
		current.release();
		kill();
		return;
	}
	
	@Override
	public void statmigration() {
		//Stats.antMigration(QueryAnt.class, 1);
	}

}
