/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package ants;

import support.IBlatAntNode;
import support.NetworkSystem;
import support.Stats;
import alpha.Entry;

public class UpdateNeighborsAnt extends Ant {

	private String target;
	private Entry local = null;
	private int step = 0;

	public UpdateNeighborsAnt(String current, String target) {
		super(current);
		local = new Entry(current);
		this.target = target;
	}

	public void antstep() {

		if (!NetworkSystem.isNodeAlive(current_)) {
			// We are on a dead node
			kill();
			return;
		}
		
		switch (step) {
		case 0:
			current.acquire();
			((IBlatAntNode) current).getInfosTo(local);
			local.setRemoteTime(current.getTime());
			current.release();
			migrate(target);
			step = 1;
			break;
		case 1:
			if (current_.equals(target)) {
				current.acquire();
				((IBlatAntNode) current).updateNeighbors(source, local);
				current.release();
			}
			step = 5;
			kill();
			break;
		}
	}

	public void reset(String current) {
		super.reset(current);
	}

	
	@Override
	public void statmigration() {
		Stats.antMigration(UpdateNeighborsAnt.class, local.getNeighborsCount());
	}

}
