/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package gnutella;

import node.GnutellaNode;
import support.NetworkSystem;
import support.Stats;
import ants.Ant;

public class GnutellaAnt extends Ant {
	
	/*  Supported messages are just PING and PONG, because we are just interested
		in the overlay construction and maintainance */
	public enum TYPE { PING, PONG, /* PUSH, QUERY, QUERYHIT */}

	private TYPE type;
	private String uuid;
	private int ttl;
	private int hops;
	private String target;
	private String origin;
	private String payload;

	public GnutellaAnt(String from, String target, TYPE type, String uuid, int ttl, int hops, String payload) {
		super(target);
		this.type = type;
		this.uuid = uuid;
		this.ttl = ttl;
		this.hops = hops;
		this.origin = from;
		this.target = target;
		this.payload = payload;
		setScheduleAfter(NetworkSystem.getRTT(from, target));
	}

	@Override
	public void antstep() {
		if ((current == null) || !NetworkSystem.isNodeAlive(current_)) {
			// We are on a dead node
			kill();
			return;
		}
		
		if (current_.equals(target)) {
			current.acquire();
			if (type == TYPE.PING)
				((GnutellaNode) current).onPing(origin, uuid, ttl, hops, target);
			else
				((GnutellaNode) current).onPong(origin, uuid, ttl, hops, target, payload);
			current.release();
		}
		kill();
		return;
		
		
	}

	@Override
	public void statmigration() {
		if (type == TYPE.PING)
			Stats.gnutellaPing();
		else
			Stats.gnutellaPong();
	}

}
