/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  BlåtAnt Algorithm implementation
 *  
 *  Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
package network;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;

import node.NodeRunner;
import support.INetworkNode;
import support.NetworkSystem;
import support.Parameters;

public class NodeSystem {

	public static Iterator<INetworkNode> iterator;

	private static List<NodeRunner> runners;

	private static CyclicBarrier b;

	private static int threadcount;

	public static void init(int count, int threads) {
		threadcount = threads;
		b = new CyclicBarrier(threads + 1);
		runners = new ArrayList<NodeRunner>();
		initRunners();
	}

	public static void initRunners() {
		if (threadcount == 0)
			return;
		runners = new ArrayList<NodeRunner>(threadcount);
		for (int i = 0; i < threadcount; i++) {
			runners.add(new NodeRunner(b));
		}
		for (NodeRunner n : runners) {
			new Thread(n).start();
		}
	}

	public static void multithreadStep() {
		if (!Parameters.DO_MULTITHREAD) {
			NetworkSystem.step();
			return;
		}
		iterator = NetworkSystem.getNetworkNodes().iterator();
		for (NodeRunner n : runners) {
			n.go();
		}
		try {
			b.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (BrokenBarrierException e) {
			e.printStackTrace();
		}
	}

	public static INetworkNode getNextNode() {
		synchronized(iterator) {
			if (iterator.hasNext())
				return iterator.next();
			else
				return null;
		}
	}


}
