/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package network;

import java.util.Set;

public class RTT {
	
	private static IRTT instance;

	public static void setInstance(IRTT rtt) {
		instance = rtt;
	}
	
	public static void initRTT(boolean directed, String unetpath) {
		if (unetpath == null) return;
		if (unetpath.endsWith(".yaml")) {
			setInstance(new RTTCacheYaml());
		} else {
			setInstance(new RTTCacheNormal());
		}
		instance.initRTT(directed, unetpath);
	}

	public static Set<String> getNodePool() {
		if (instance != null)
			return instance.getNodePool();
		else
			return null;
	}

	public static long getPathWeight(String x, String y) {
		if (instance == null) return 1;
		return instance.getPathWeight(x, y);
	}

	public static long mstoiter(long ms) {
		if (instance == null) return ms;
		return instance.mstoiter(ms);
	}

	public static long itertoms(long iter) {
		if (instance == null) return iter;
		return instance.itertoms(iter);
	}

	public static Double mstoiterd(long ms) {
		return new Double(instance.mstoiter(ms));
	}

}
