/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package pheromone;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import network.RTT;
import support.Parameters;

public class Pheromone {

	private HashMap<String, Trail> trails;
	
	private String name;
	private final Double minvalue = 0.019810710;
	@SuppressWarnings("unused")
	private String id;
	private List<String> d = new LinkedList<String>();
	
	private double[] values;
	
	private static double[] betavalues;
	private static double[] gammavalues;

	public Pheromone(String trailname, String nodeId) {
		trails = new HashMap<String, Trail>(200);
		id = nodeId;
		name = trailname;
		
		if (trailname.equals("beta")) {
			if (betavalues == null) {
				Long totaldecayiter = RTT.mstoiter(Parameters.BETA_DECAY_MS);
				if (betavalues == null) {
					double decay = Math.pow((minvalue / Parameters.INITIAL_CONCENTRATION), (1.0 / new Double(totaldecayiter)));
					betavalues = new double[totaldecayiter.intValue()];
					double initial = Parameters.INITIAL_CONCENTRATION;
					for (int i = 0; i < totaldecayiter; i++) {
						betavalues[i] = initial * Math.pow(decay,i);
					}
				}
			}
			values = betavalues;
		} else {
			if (gammavalues == null) {
				Long totaldecayiter = RTT.mstoiter(Parameters.GAMMA_DECAY_MS);
				if (gammavalues == null) {
					double decay = Math.pow((minvalue / Parameters.INITIAL_CONCENTRATION), (1.0 / new Double(totaldecayiter)));
					gammavalues = new double[totaldecayiter.intValue()];
					double initial = Parameters.INITIAL_CONCENTRATION;
					for (int i = 0; i < totaldecayiter; i++) {
						gammavalues[i] = initial * Math.pow(decay,i);
					}
				}
			}
			values = gammavalues;
		}

	}

	public synchronized void clear(String trail) {
		trails.remove(new String(trail));
	}

	public String getName() {
		return name;
	}

	public synchronized List<String> decay() {
		d.clear();
		Iterator<Trail> it = trails.values().iterator();
		while(it.hasNext()) {
			Trail tra = it.next();
			if (tra.decay() >=  values.length) {
				d.add(tra.name);
				it.remove();
			}
		}
		/*for (Trail trail : trails.values()) {
			if (trail.decay() >=  values.length) {
				d.add(trail.name);
			}
		}
		for (String t : d) {
			trails.remove(t);
		}*/
		return d;
	}

	public synchronized void fill(String trail) {
		Trail t = trails.get(trail);
		if (t == null) {
			t = new Trail(trail);
			trails.put(trail, t);
		} else {
			t.fill();
		}
	}

	public synchronized void remove(String trail) {
		if (trail == null)
			return;
		trails.remove(trail);
	}

	public synchronized double senseTrail(String trail) {
		Trail t = trails.get(trail);
		if (t == null)
			return 0;
		else
			return values[t.value];
	}

	public void clearAll() {
		trails.clear();
		d.clear();
	}

}
