/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package remote;

import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import messages.SmartGridQuery;
import bebop.buffer.IBuffer;
import bebop.client.Client;
import bebop.lib.IMessage;
import bebop.lib.Message;
import bebop.server.Server;

public class RemoteController implements Observer {

	private IRemoteControllableNetwork net;
	private IBuffer buffer;

	public RemoteController(int port, IRemoteControllableNetwork net) {
		this.net = net;
		this.buffer = Server.start(this, port);
		System.out
				.format(
						"Remote control BeBop Server listening on port %d, controlling %s\n",
						port, net.toString());
	}

	public void update(Observable source, Object data) {
		if (source == this.buffer) {
			getMessage();
		}
	}

	@SuppressWarnings("unchecked")
	private synchronized void getMessage() {
		IMessage msg = buffer.popMessage();
		if (msg != null) {
			//System.out.println("Received new message: " + msg.toString());
			Map<String, Object> contents = msg.getContents();
			if (contents.containsKey("command")) {
				String command = (String) contents.get("command");
				if (command.equals("attachNode")) {
					String node = (String) contents.get("id");
					String hook = (String) contents.get("hook");
					net.attachNode(node, hook);
				} else if (command.equals("detachNode")) {
					String node = (String) contents.get("id");
					net.detachNode(node);
				} else if (command.equals("removeNode")) {
					String node = (String) contents.get("id");
					net.removeNode(node);
				} else if (command.equals("query")) {
					String node = (String) contents.get("id");
					Map<String,Object> query = (Map<String,Object>) contents.get("query");
					SmartGridQuery sgq = new SmartGridQuery(node, query);
					net.query(node, sgq);
				} else if (command.equals("getNodeList")) {
					String address = (String) contents.get("address");
					String recipient = (String) contents.get("recipient");
					List<String> l = net.getNodeList();
					Map<String, Object> m = msg.getContents();
					m.put("result", l);
					Message reply = new Message(recipient, m);
					try {
						Client.sendMessage(address, reply);
					} catch (Exception e) {
						//e.printStackTrace();
					}
				} else if (command.equals("getNeighbors")) {
					String node = (String) contents.get("id");
					String address = (String) contents.get("address");
					String recipient = (String) contents.get("recipient");
					List<String> l = net.getNeighbors(node);
					Map<String, Object> m = msg.getContents();
					m.put("result", l);
					Message reply = new Message(recipient, m);
					try {
						Client.sendMessage(address, reply);
					} catch (Exception e) {
						//e.printStackTrace();
					}
				}
			}
		}
	}

}
