/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package services;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import services.dss.profile.ResourceProfile;
import support.NetworkSystem;
import support.Parameters;

public class ProfileCacheMap extends LinkedHashMap<String,ResourceProfile> {
	
	private static final long serialVersionUID = 1L;
	
	private Map<String,Long> insertionTimes = new HashMap<String,Long>();

     @SuppressWarnings("unchecked")
	protected boolean removeEldestEntry(Map.Entry eldest) {
        return size() > Parameters.REPLICATION_CACHE_SIZE;
     }
     
     public ResourceProfile put(String key, ResourceProfile value) {
    	insertionTimes.put(key, NetworkSystem.getGlobalTimeMs());
		return super.put(key, value);	 
     }
     
     public void cleanup() {
    	if (Parameters.CACHE_TIME_MS < 0) return;
    	ArrayList<String> keys = new ArrayList<String>();
    	keys.addAll(insertionTimes.keySet());
    	Long time = NetworkSystem.getGlobalTimeMs();
    	for (String k : keys) {
    		if (time - insertionTimes.get(k) > Parameters.CACHE_TIME_MS) {
    			insertionTimes.remove(k);
    			this.remove(k);
    		}
    	} 
     }
     
     


}
