/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package services.dss;

import java.util.ArrayList;
import java.util.Map;
import java.util.UUID;

import messages.Query;
import services.dss.JobStatusManager.Status;
import services.dss.profile.ResourceProfile;
import support.INetworkNode;
import support.Parameters;
import ants.AntSystem;
import ants.DSSAcceptAnt;
import ants.DSSAssignAnt;
import ants.DSSInformAnt;
import ants.DSSRequestAnt;
import ants.DSSStatusAnt;
import ants.ISSDSSInformAnt;
import ants.ISSDSSRequestAnt;

public class Messaging {
	
	public static void sendAccept(INetworkNode d,  String jobid, Long eta, String target) {
		DSSStats.acceptStarted();
		AntSystem.schedule(new DSSAcceptAnt(d.getIdentifier(), target, jobid, eta));
	}

	@SuppressWarnings("unchecked")
	public static void sendInform(INetworkNode d, ResourceProfile profile, Long eta) {
		DSSStats.informStarted();
		if (Parameters.ISS_ENABLED) {
			AntSystem.schedule(new ISSDSSInformAnt(d.getIdentifier(), d.getIdentifier(), d.getIdentifier(),
					new Query(d.getIdentifier(), profile), Parameters.DSS_INFORM_TTL, false, new ArrayList<String>(), eta));
		} else {
			AntSystem.schedule(new DSSInformAnt(d.getIdentifier(), d.getIdentifier(), d.getIdentifier(), (Map<String, Object>) profile.toBaseValue(), eta, Parameters.DSS_INFORM_TTL, UUID.randomUUID().toString()));
		}
	}

	public static void sendRequest(INetworkNode d, ResourceProfile profile) {
		DSSStats.requestStarted();
		if (Parameters.ISS_ENABLED) {
			AntSystem.schedule(new ISSDSSRequestAnt(d.getIdentifier(), d.getIdentifier(), d.getIdentifier(),
					new Query(d.getIdentifier(), profile), Parameters.DSS_INFORM_TTL, false, new ArrayList<String>()));
		} else {
			AntSystem.schedule(new DSSRequestAnt(d.getIdentifier(), d.getIdentifier(), d.getIdentifier(), profile, Parameters.SEARCH_LIMIT_TTL));
		}
	}
	
	@SuppressWarnings("unchecked")
	public static void sendAssign(INetworkNode d, ResourceProfile job, String target) {
		DSSStats.assignStarted();
		AntSystem.schedule(new DSSAssignAnt(d.getIdentifier(), target, (Map<String, Object>) job.toBaseValue()));
	}
	
	public static void sendStatus(INetworkNode d, String jobid, String target, Status s, Double timestamp) {
		DSSStats.statusStarted();
		AntSystem.schedule(new DSSStatusAnt(d.getIdentifier(), target, jobid, s, d.getIdentifier(), timestamp));
	}

	public static void sendStatus(INetworkNode d, String jobid, String target, Status s, Double timestamp, long remainingTime, long drift) {
		DSSStats.statusStarted();
		AntSystem.schedule(new DSSStatusAnt(d.getIdentifier(), target, jobid, s, d.getIdentifier(), timestamp, remainingTime, drift));
	}


}
