/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package services.dss.monitor;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;

import services.dss.DynamicSchedulingService;
import services.dss.profile.ResourceProfile;
import support.INetworkNode;
import algo.Algorithm;

public class NodeSchedulingPanel extends JPanel implements ActionListener, MouseListener {
	
	private static final long serialVersionUID = 3142839805675497782L;
	private JLabel label;
	private JLabel schedulerLabel;
	private JButton addJobButton;
	private INetworkNode d;
	private DynamicSchedulingService dss;
	
	public NodeSchedulingPanel(final INetworkNode d, DynamicSchedulingService dss) {
		
		this.setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
		addJobButton = new JButton("+");
		addJobButton.addActionListener(this);
		addJobButton.setToolTipText("Submit a job to this node");
		this.add(addJobButton);
		
		addJobButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				Algorithm.addRandomJob(Algorithm.jobp, d);
			}
			
		});
		
		
		label = new JLabel();
		label.setBorder(new EmptyBorder(0,10,0,10));
		label.setMaximumSize(new Dimension(80, 20));
		label.setMinimumSize(new Dimension(80, 20));
		label.setPreferredSize(new Dimension(80, 20));
		this.add(label);
		
		schedulerLabel = new JLabel();
		schedulerLabel.setFont(getFont().deriveFont((float) 8));
		schedulerLabel.setBorder(new EmptyBorder(0,10,0,10));
		schedulerLabel.setMaximumSize(new Dimension(70, 30));
		schedulerLabel.setMinimumSize(new Dimension(70, 30));
		schedulerLabel.setPreferredSize(new Dimension(70, 30));
		this.add(schedulerLabel);
		
		this.addMouseListener(this);
		
		this.d = d;
		setLabelName();
		updateNodeData();
		this.add(new SchedulerMonitorWidget(dss));		
		this.setBorder(new CompoundBorder
			      (BorderFactory.createRaisedBevelBorder(),
			       new EmptyBorder(10,10,10,10)));
	}
	
	public void update() {
		updateNodeData();
	}
	
	private void setLabelName() {
		label.setText(d.getIdentifier());
	}
	
	public INetworkNode getDaemon() {
		return d;
	}
	
	public DynamicSchedulingService getDSS() {
		return dss;
	}
	
	private void updateNodeData() {
		if (dss == null) dss = d.getDSSService();
		ResourceProfile profile = null;
		if (dss == null) { 
		} else {
			profile = d.getProfile();
			int offers = dss.getAcceptOfferManager().getAcceptingQueueSize();
			if (offers > 0) {
				label.setForeground(Color.RED);
			} else
				label.setForeground(Color.BLACK);
			if (d.getProfile().matches(DSSMonitorStats.getHighlightedJob()))
				schedulerLabel.setForeground(Color.blue);
			else
				schedulerLabel.setForeground(Color.black);
			schedulerLabel.setText(String.format("<html>%s<br>%d/%d</html>",dss.getScheduler().getSchedulerName(), dss.getScheduler().getScheduleSize(), offers));
		}
		String profiletable;
		
		if (profile == null)
			profiletable = String.format("<html><table border=\"0\">" +
					"<tr><td><b>Identifier</b>:</td><td>%s</td></tr></table></html>", d.getIdentifier());
		else 
			profiletable = String.format("<html><table border=\"0\">" +
				"<tr><td><b>Identifier</b>:</td><td>%s</td></tr>" +
				"<tr><td><b>Arch</b>:</td><td>%s</td></tr>" +
				"<tr><td><b>OS</b>:</td><td>%s</td></tr>" +
				"<tr><td><b>Version</b>:</td><td>%d</td></tr>" +
				"<tr><td><b>Memory</b>:</td><td>%d</td></tr>" +
				"<tr><td><b>Disk</b>:</td><td>%d</td></tr>" +
				"<tr><td><b>CPUSpeed</b>:</td><td>%3.2f</td></tr>" +
				"</table></html>", 
				d.getIdentifier(), profile.arch, profile.os, profile.minimumVersion, 
				profile.memory, profile.disk, profile.cpuspeed);

		label.setToolTipText(profiletable);
	}

	public void actionPerformed(ActionEvent arg0) {
		
	}

	public void mouseClicked(MouseEvent arg0) {
		if (arg0.getClickCount() == 2) {
			System.out.println("Node "+d.getIdentifier()+" stats:");
			System.out.println("========================================================");
			System.out.println("Profile: "+d.getProfile());
			System.out.println("Scheduled task:");
			List<String> tasks = dss.getScheduler().getJobs();
			int i = 1;
			for (String j : tasks) {
				ResourceProfile p = dss.getScheduler().getProfile(j);
				System.out.println("\t"+i+". - Running? "+p.isRunning()+" Ready? "+p.isReady()+" Locked? "+p.isLocked()+" : "+p.toString());
				i++;
			}
			System.out.println("========================================================");
		}
	}

	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	
}
