/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package services.dss.profile;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import support.INetworkNode;
import support.NetworkSystem;
import support.Parameters;


public class ResourceProfile {

	private static final long serialVersionUID = -7315971827590151109L;

	public String arch;
	
	public List<Long> actualCosts;
	
	public List<Long> promisedCosts;
	
	public List<Long> delays;

	public Long disk;
	private Long estimatedRunningTime;
	
	public Long actualRunningTime = -1l;
	
	public String id;
	public String pool;
	public Integer poolsize = 1;
	public Long memory;
	public Long minimumVersion;
	public double cpuspeed;
	
	public String os;

	private boolean runningFlag = false;
	private boolean isReady = false;
	public boolean isRunnable = false;
	
	
	public Long startedAt = 0l;
	public Long finishedAt = 0l;
	public long drift = 0l;
	public Long createdAt = 0l;

	public Long remainingTime = 0l;
	
	public Long deadline = 0l;

	public int assigns = 0;
	
	public int submitRetries = 0;
	
	public Long tempSlack = 0l;
		
	public Long reservationStart = 0l;
	public Long reservationEnd = 0l;
	
	public Long startDelay = 0l;
	
	public Long tempDelay = 0l;

	public Long informs = 0l;

	public INetworkNode currentNode = null;

	public String initiator = null;

	public ResourceProfile() {
		createdAt = NetworkSystem.getGlobalTimeMs();
	}

	@SuppressWarnings("unchecked")
	public ResourceProfile(Map<String, Object> data) {
		id = (String) getIfSet(data, "id", "");
		pool = (String) getIfSet(data, "pool", "");
		poolsize = (Integer) getIfSet(data, "poolsize", 1);
		arch = (String) getIfSet(data, "arch", "");
		os = (String) getIfSet(data, "os", "");
		minimumVersion = (Long) getIfSet(data, "minimumVersion", 0l);
		memory = (Long) getIfSet(data, "memory", 0l);
		disk = (Long) getIfSet(data, "disk", 0l);
		estimatedRunningTime = (Long) getIfSet(data, "estimatedRunningTime", 0l);
		actualRunningTime = (Long) getIfSet(data, "actualRunningTime",  -1l);
		createdAt = (Long) getIfSet(data, "createdAt", 0l);
		initiator = (String) getIfSet(data, "initiator", "");
		assigns = (Integer) getIfSet(data, "assigns", 0);
		deadline = (Long) getIfSet(data, "deadline", 0l);
		reservationStart = (Long) getIfSet(data, "reservationStart", 0l);
		reservationEnd = (Long) getIfSet(data, "reservationEnd", 0l);
		actualCosts = (List<Long>) getIfSet(data, "actualCosts", new ArrayList<Long>());
		promisedCosts = (List<Long>) getIfSet(data, "promisedCosts", new ArrayList<Long>());
		informs = (Long) getIfSet(data, "informs", 0l);
		delays = (List<Long>) getIfSet(data, "delays", new ArrayList<Long>());
	}
	
	public long getEstimatedRunningTime(double cpuspeed) {
		return (long) (this.estimatedRunningTime / cpuspeed); 
	}
	
	public void setEstimatedRunningTime(long ert) {
		this.estimatedRunningTime = ert;
	}

	public boolean equals(Object o) {
		if (o == null)
			return false;
		if (!(o instanceof ResourceProfile))
			return false;
		return o.toString().equals(toString());
	}

	private Object getIfSet(Map<String, Object> data, String key, Object d) {
		if (data.containsKey(key))
			return data.get(key);
		else
			return d;
	}

	public int hashCode() {
		return toString().hashCode();
	}

	public boolean matches(ResourceProfile o) {
		if (o == null) return false;
		if (!Parameters.SKIP_ARCH_MATCH && !arch.equals(o.arch))
			return false;
		if (!Parameters.SKIP_OS_MATCH && !os.equals(o.os))
			return false;
		if (!Parameters.SKIP_VER_MATCH && (minimumVersion < o.minimumVersion))
			return false;
		if (!Parameters.SKIP_MEM_MATCH && (memory < o.memory))
			return false;
		if (!Parameters.SKIP_DISK_MATCH && (disk < o.disk))
			return false;
		return true;
	}
	
	public boolean isWithinSimilarityBoundsOf(ResourceProfile o) {
		// o is the requested profile
		if (!arch.equals(o.arch))
			return false;
		if (!os.equals(o.os))
			return false;
		if (minimumVersion < o.minimumVersion)
			return false;
		// this object is within similarity bounds of o if
		//  - this.disk is within the bounds of o.disk
		//  - this.memory is within the bounds of o.memory
		if ((disk < o.disk/2) || (disk > 2*o.disk))
			return false;
		if ((memory < o.memory/2) || (memory > 2*o.memory))
			return false;
		return true;
	}


	public Object toBaseValue() {
		Map<String, Object> data = new HashMap<String, Object>();
		data.put("id", id);
		data.put("arch", arch);
		data.put("os", os);
		data.put("initiator", initiator);
		data.put("minimumVersion", minimumVersion);
		data.put("memory", memory);
		data.put("disk", disk);
		data.put("estimatedRunningTime", estimatedRunningTime);
		data.put("createdAt", createdAt);
		data.put("assigns", assigns);
		data.put("deadline", deadline);
		data.put("reservationStart", reservationStart);
		data.put("reservationEnd", reservationEnd);
		data.put("actualCosts", actualCosts);
		data.put("promisedCosts", promisedCosts);
		data.put("informs", informs);
		data.put("delays", delays);
		data.put("pool", pool);
		data.put("poolsize", poolsize);
		data.put("actualRunningTime", actualRunningTime);
		return data;
	}
	
	@SuppressWarnings("unchecked")
	public Map<String,Object> toMap() {
		return (Map<String,Object>)toBaseValue();
	}
	
	public ResourceProfile copy() {
		ResourceProfile copy = new ResourceProfile();
		copy.id = id;
		copy.arch = arch;
		copy.os = os;
		copy.initiator = initiator;
		copy.minimumVersion = minimumVersion;
		copy.memory = memory;
		copy.disk = disk;
		copy.estimatedRunningTime = estimatedRunningTime;
		copy.createdAt = createdAt;
		copy.assigns = assigns;
		copy.deadline = deadline;
		copy.reservationStart = reservationStart;
		copy.reservationEnd = reservationEnd;
		copy.actualCosts = new ArrayList<Long>();
		copy.actualCosts.addAll(actualCosts);
		copy.promisedCosts = new ArrayList<Long>();
		copy.promisedCosts.addAll(promisedCosts);
		copy.delays = new ArrayList<Long>();
		copy.delays.addAll(delays);
		copy.informs = informs;
		copy.pool = pool;
		copy.poolsize = poolsize;
		copy.actualRunningTime = actualRunningTime;
		return copy;
	}

	public String toString() {
		return String.format("{%s, %s, %d, %d, %s, %s, %d, %d, %d, %d, %d, %d, %d, %d, %d}", id, pool, poolsize, createdAt, arch, os,
				minimumVersion, memory, disk, estimatedRunningTime, actualRunningTime, assigns, deadline, reservationStart, reservationEnd);
	}
	
	public void setPool(String pool) {
		this.pool = pool;
	}
	
	public boolean setReady() {
		if (this.isReady) return false;
		this.isReady = true;
		return true;
	}
	
	public boolean isReady() {
		return isReady;
	}
	
	public boolean isLocked() {
		return runningFlag || isReady;
	}
	
	public boolean isRunning() {
		return runningFlag;
	}
	
	public void setRunning() {
		runningFlag = true;
	}
	

}
