/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  BlåtAnt Algorithm implementation
 *  
 *  Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
package smartgrid;

import node.BlatAntSNode;
import support.NetworkSystem;
import support.Parameters;
import support.Randomizer;
import support.Stats;
import algo.Algorithm;
import ants.AntSystem;
import ants.BalancingAnt;
import ants.DiscoverySAnt;


public class SSLSimulator extends Algorithm {

	public static void main(String[] args) {
		new SSLSimulator(args);
	}

	protected synchronized void addNode(String id, String hook) {
		if (!NetworkSystem.existNode(id)) {
			BlatAntSNode node = new BlatAntSNode(new String(id), firstActive());
			NetworkSystem.addNode(node);
		}
		if (Parameters.NOTIFY)
			System.out.format("Created %s\n", id);
		Stats.newNode();
		NetworkSystem.initiateUserLink(new String(id), new String(hook));
	}

	public SSLSimulator(String[] args) {
		setupCLI(args);
		parseCLI();

		System.out.format("---------------------------------------------\n");
		System.out.println("SSL Simulator (Blatant-S)");
		System.out.println("(C) 2009 Amos Brocco <amos.brocco@unifr.ch>");
		System.out.format("---------------------------------------------\n");

		printConfiguration();

		System.out.format("NODES = %d\n", NetworkSystem.getNodeCount());
		System.out.format("---------------------------------------------\n");
		
		this.execute();
	}

	protected void generateAnts() {
		for (String n : NetworkSystem.getNodes()) {
			generateAnt(n);
			if (Parameters.BALANCING) {
				if (NetworkSystem.getNode(n).getLoadManagementService()
						.getLoad() > 50)
					AntSystem.schedule(new BalancingAnt(n));
			}
		}
	}

	public void generateAnt(String n) {
		if (Randomizer.pop() < Parameters.ANT_BIRTH_PROBABILITY) {
			if (!Parameters.DISABLE_BLATANT)
				AntSystem.schedule(new DiscoverySAnt(n));
		}
	}

	private boolean active = true;

	private boolean firstActive() {
		if (NetworkSystem.getNodeCount() == 0) {
			active = true;
		}
		if (active) {
			active = false;
			return true;
		}
		return false;
	}

	@Override
	protected void algorithmStep() {
		// TODO Auto-generated method stub
		
	}
	
}
