/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  BlåtAnt Algorithm implementation
 *  
 *  Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
package support;

import java.util.Random;

public class Randomizer extends Random {

	private static final long serialVersionUID = 1121471786417661403L;

	static Random rnd;
	
	private Random rrandom;
	
	static double[] doubles = new double[65536];
	static int[] ints = new int[65536];
	
	static int currentd;
	static int currenti;

	public Randomizer(long seed) {
		super(seed);
		rrandom = new Random(seed);
		refresh();
	}

	public static void init(long seed) {
		if (seed == -1)
			seed = System.currentTimeMillis();
		Parameters.RANDOM_SEED = seed;
		if (Parameters.NO_RANDOM_CACHE)
			rnd = new Random(seed);
		else
			rnd = new Randomizer(seed);
	}
	
	private void refresh() {
		for (int i = 0; i < doubles.length; i++) {
			doubles[i] = rrandom.nextDouble();
		}
		currentd = 0;
		for (int i = 0; i < ints.length; i++) {
			ints[i] = rrandom.nextInt();
		}
		currenti = 0;
	}

	public static double pop() {
		return rnd.nextDouble();
	}
	
	public double nextDouble() {
		if (currentd == doubles.length) {
			currentd = 0;
		}
		return doubles[currentd++];
	}
	
	public int nextInt() {
		if (currenti == ints.length) {
			currenti = 0;
		}
		return ints[currenti++];
	}

	public static Random getRandom() {
		return rnd;
	}

}
