/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  BlåtAnt Algorithm implementation
 *  
 *  Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
package support;

import java.util.HashSet;
import java.util.Hashtable;

import messages.Message;
import util.Dijkstra;
import util.Graph;
import ants.BalancingAnt;
import ants.ConstructionLinkAnt;
import ants.DiscoveryAnt;
import ants.DiscoverySAnt;
import ants.ISSProactiveQueryAnt;
import ants.ISSQueryAnt;
import ants.OptimizationLinkAnt;
import ants.PingAnt;
import ants.QueryAnt;
import ants.UnlinkAnt;
import ants.UpdateNeighborsAnt;

public class Stats {

	public static Hashtable<String,FloodJob> searchjobs = new Hashtable<String,FloodJob>();
	
	public static int remove = 0;
	public static int add = 0;
	public static int deadremove = 0;
	public static int birth = 0;
	public static int migrations = 0;
	public static int respawn = 0;
	/*
	 * public static int newNodes = 0; public static int delNodes = 0;
	 */
	public static int balancing = 0;
	public static int forwardings = 0;
	public static Hashtable<String, String> userlink = new Hashtable<String, String>();
	public static Hashtable<String, Double> loads = new Hashtable<String, Double>();
	
	public static int replicas = 0;
	public static int rndWalker = 0;
	public static int callback = 0;
	public static int neighbors = 0;
	public static boolean partitioned = false;
	public static int pings = 0;
	public static int largestpartition = 0;
	public static int smallestpartition = 0;
	public static int slow = 0;
	public static int loss = 0;
	public static int newnodes = 0;
	public static int leavenodes = 0;
	public static int remnodes = 0;
	public static int isshops = 0;
	public static int prohops = 0;
	public static int prohits = 0;
	public static int prostarted = 0;
	public static int proreplies = 0;
	public static int isspings = 0;
	
	public static int issmerges = 0;
	public static int issmergescost = 0;
	
	public static int replicantstarted = 0;
	public static int replicanthops = 0;
	
	public static int treplicantstarted = 0;
	public static int treplicanthops = 0;
	
	public static int replicanthit = 0;
	public static int treplicanthit = 0;
	
	public static int publish = 0;
	public static int tpublish = 0;
	
	public static int nstokens = 0;
	
	
	public static int gnuping = 0;
	public static int gnupong = 0;
	public static int gnuconnect = 0;
	
	public static int tgnuping = 0;
	public static int tgnupong = 0;
	public static int tgnuconnect = 0;
	
	public static Long initiatedconn = 0l;
	public static Long triedconn = 0l;
	public static Long wrongconn = 0l;
	public static Long otriedconn = 0l;
	public static long oinitiatedconn = 0l;
	public static Long owrongconn = 0l;
	
	public static HashSet<String> queryid = new HashSet<String>();
	
	public static Hashtable<String,Long> antMigrations = new Hashtable<String,Long>();
	public static Hashtable<String,Long> antMigrationsTokens = new Hashtable<String,Long>();
	
	public static void initiatedconn() {
		synchronized(initiatedconn) {
			initiatedconn++;
			oinitiatedconn++;
		}
	}
	
	public static double totconntime = 0;
	public static double totconnect = 0;
	
	public static double totdisconntime = 0;
	public static double totdisconnect = 0;
	
	public synchronized static void connectionTime(long time) {
		totconntime += time;
		totconnect += 1;
	}
	
	public synchronized static void disconnectionTime(long time) {
		totdisconntime += time;
		totdisconnect += 1;
	}
	
	public static void triedconnection(String from, String to) {
		if (Parameters.CONN_ERROR) {
			Graph g = NetworkSystem.getGraph();
			long dist = Dijkstra.getShortestPath(g, from, to).size();
			if (dist > 2 && dist < (2*Parameters.D - 1)) {
				synchronized(triedconn) {
					triedconn++;
					wrongconn++;
					otriedconn++;
					owrongconn++;
				}
			} else {
				synchronized(triedconn) {
					triedconn++;
					otriedconn++;
				}
			}
		}
	}

	public static void resetAntMigrations() {
		antMigrations.clear();
		antMigrationsTokens.clear();
		Class<?>[] antClasses = { BalancingAnt.class, ConstructionLinkAnt.class,
				DiscoveryAnt.class, DiscoverySAnt.class, ISSProactiveQueryAnt.class, 
				ISSQueryAnt.class, OptimizationLinkAnt.class, PingAnt.class, QueryAnt.class,
				UnlinkAnt.class, UpdateNeighborsAnt.class
		};
		
		for (Class<?> c : antClasses) {
			antMigrations.put(c.getName(), new Long(0));
			antMigrationsTokens.put(c.getName(), new Long(0));
		}
	}
	
	public static synchronized void antMigration(Class<?> type, int cost) {
		long val = antMigrations.get(type.getName()) + 1;
		antMigrations.put(type.getName(), val);
		long valcost = antMigrationsTokens.get(type.getName()) + cost;
		antMigrationsTokens.put(type.getName(), valcost);
	}
	
	public static int merges = 0;

	public static long nontokens = 0;

	public static void resetNodes() {
		newnodes = 0;
		leavenodes = 0;
		remnodes = 0;
	}
	
	public synchronized static void issMerge(int cost) {
		issmerges++;
		issmergescost += cost;
	}

	public synchronized static void newNode() {
		newnodes++;
	}

	public synchronized static void leaveNode() {
		leavenodes++;
	}

	public synchronized static void remNode() {
		remnodes++;
	}

	public static void tellLoad(String node, Double load) {
		loads.put(node, load);
	}

	public static double loadVariance() {
		if (!Parameters.BALANCING)
			return 0;

		if (loads.isEmpty()) {
			return 0;
		}
		double total = 0;
		for (String key : loads.keySet()) {
			total += loads.get(key);
		}
		double mean = total / ((double) loads.size());
		double variance = 0;
		for (String key : loads.keySet()) {
			variance += Math.abs(loads.get(key) - mean);
		}

		variance = variance / ((double) loads.size());
		return variance;
	}

	public synchronized static void addLink() {
		add++;
	}

	public synchronized static void deadRemove() {
		deadremove++;
	}

	public synchronized static void removeLink() {
		remove++;
	}

	public static void reset() {
		//System.out.println("<--- RESET --->");
		
		triedconn = 0l;
		wrongconn = 0l;
		initiatedconn = 0l;
		
		resetAntMigrations();
		remove = 0;
		deadremove = 0;
		add = 0;
		birth = 0;
		respawn = 0;
		balancing = 0;
		migrations = 0;
		pings = 0;
		forwardings = 0;
		slow = 0;
		loss = 0;
		merges = 0;
		issmerges = 0;
		nstokens = 0;
		issmergescost = 0;
		prohops = 0;
		prohits = 0;
		prostarted = 0;
		proreplies = 0;
		isspings = 0;
		replicantstarted = 0;
		replicanthops = 0;
		replicanthit = 0;
		publish = 0;
		
		gnuping = 0;
		gnupong = 0;
		gnuconnect = 0;
		nontokens = 0;
	}

	public static boolean isUserLink(String id, String identifier) {
		if (userlink.containsKey(id + "-" + identifier))
			return true;
		return false;
	}

	public static void userLink(String id, String identifier) {
		userlink.put(id + "-" + identifier, "yep");
	}

	public static synchronized void antBirth() {
		birth++;
	}

	public static synchronized void antRespawn() {
		respawn++;
	}

	public static synchronized void balancing() {
		balancing++;
	}

	public static synchronized void notifyForwarding() {
		forwardings++;
	}

	public static synchronized void randomWalker() {
		rndWalker++;
	}

	public static synchronized void callback() {
		callback++;
	}

	public static synchronized void randomWalkerKill() {
		rndWalker--;
	}

	public static synchronized void addNeighbor() {
		neighbors++;
	}

	public static synchronized void partitioned() {
		partitioned = true;
	}

	public static synchronized void unpartitioned() {
		partitioned = false;
	}

	public static synchronized void ping() {
		pings++;
	}

	public static synchronized void partition(int nodesvisited) {
		if (nodesvisited > largestpartition)
			largestpartition = nodesvisited;
		if (nodesvisited < smallestpartition)
			smallestpartition = nodesvisited;

	}

	public static synchronized void slow() {
		slow++;
	}

	public static synchronized void loss() {
		loss++;
	}

	public static synchronized void messageMatch(Message query, String match) {
		if (searchjobs.containsKey(query.getIdentifier()))
			searchjobs.get(query.getIdentifier()).hit(match);
	}
	
	public static synchronized void messageHop(Message query, String from, String to) {
		if (searchjobs.containsKey(query.getIdentifier()))
			searchjobs.get(query.getIdentifier()).hop(from, to);
	}

	public static void clearSearchJobs() {
		isshops = 0;
		searchjobs.clear();
	}

	public static void addSearchJob(FloodJob job) {
		searchjobs.put(job.getQueryIdentifier(), job);
	}

	public static int searchPoolSize() {
		int psize = 0;
		for (FloodJob s : searchjobs.values()) {
			psize += s.getPoolSize();
		}
		return psize;
	}

	public static int searchFound() {
		int fsize = 0;
		for (FloodJob s : searchjobs.values()) {
			fsize += s.getFound();
		}
		return fsize;
	}

	public static int searchHits() {
		int hsize = 0;
		for (FloodJob s : searchjobs.values()) {
			hsize += s.getHits();
		}
		return hsize;
	}
	
	public static int searchHitsOnISS() {
		int hsize = 0;
		for (FloodJob s : searchjobs.values()) {
			hsize += s.getHitsOnISS();
		}
		return hsize;
	}

	public static int searchQueryHops() {
		int hsize = 0;
		for (FloodJob s : searchjobs.values()) {
			hsize += s.getHops();
		}
		return hsize;
	}
	
	public static int searchSuccess() {
		int hsize = 0;
		for (FloodJob s : searchjobs.values()) {
			if (s.getHits() > 0) {
				hsize++;
			}
		}
		return hsize;
	}

	public static synchronized void isshop() {
		isshops++;
	}

	public static synchronized void merge() {
		merges ++;
	}

	public static synchronized void queryMatchOnISS(Message query, String match) {
		if (searchjobs.containsKey(query.getIdentifier()))
			searchjobs.get(query.getIdentifier()).hitOnISS(match);
	}

	public static synchronized void proactiveQueryHops() {
		prohops++;
	}

	public static synchronized void prohit() {
		prohits++;
	}
	
	public static synchronized void prostarted() {
		prostarted++;
	}

	public static synchronized void proReply() {
		proreplies++;
	}

	public static synchronized void isspings() {
		isspings++;
	}

	public static synchronized void newscastTokens(int i) {
		nstokens++;
	}

	public static synchronized void replicantHop() {
		replicanthops++;
		treplicanthops++;
	}

	public static synchronized void replicantStarted() {
		replicantstarted++;
		treplicantstarted++;
	}
	
	public static synchronized void replicantHit() {
		replicanthit++;
		treplicanthit++;
	}

	public static synchronized void publish() {
		publish++;
		tpublish++;
	}

	public static synchronized void gnutellaPing() {
		gnuping++;
		tgnuping++;
	}

	public static synchronized void gnutellaPong() {
		gnupong++;
		tgnupong++;
	}
	
	public static synchronized void gnutellaConnect() {
		gnuconnect++;
		tgnuconnect++;
	}

	public static synchronized void nonTokens(long total) {
		nontokens += total;
	}

	public static long connects = 0;
	public static long disconnects = 0;
	
	public static synchronized void evDisConn() {
		connects++;
	}
	
	public static synchronized void evConn() {
		disconnects++;
	}
	
	

}
