/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  BlåtAnt Algorithm implementation
 *  
 *  Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
package syncutil;

import node.SimpleNode;
import support.IStar;
import support.NetworkSystem;
import support.Parameters;
import support.Stats;

public class TestCompute {

	public class ProgressStarFake implements IStar {

		public void show() {
		}

		public void hide() {
		}

		public synchronized void progress() {
		}

	}

	public TestCompute() {

		int threads = 6;

		//Algorithm.star = new ProgressStarFake();
		NetworkSystem.init(1000, threads);

		/* Test topology */

		SimpleNode one = new SimpleNode("1");
		SimpleNode two = new SimpleNode("2");
		SimpleNode three = new SimpleNode("3");
		SimpleNode four = new SimpleNode("4");
		SimpleNode five = new SimpleNode("5");
		SimpleNode six = new SimpleNode("6");
		SimpleNode seven = new SimpleNode("7");
		SimpleNode eight = new SimpleNode("8");
		SimpleNode nine = new SimpleNode("9");
		SimpleNode ten = new SimpleNode("10");

		/**
		 * 1 -- 2 -- 3 | | 7 -- 4 -- 5 \ | \ 6
		 * 
		 * 8 -- 9 -- 10
		 * 
		 */

		Parameters.COMPLETE_NS_LCC = true;

		/* 16 edges there */
		one.addNeighbor("2");
		two.addNeighbor("1");
		two.addNeighbor("3");
		three.addNeighbor("2");
		three.addNeighbor("5");
		two.addNeighbor("4");
		five.addNeighbor("3");
		four.addNeighbor("2");
		four.addNeighbor("5");
		five.addNeighbor("4");
		four.addNeighbor("7");
		four.addNeighbor("6");
		six.addNeighbor("4");
		six.addNeighbor("7");
		seven.addNeighbor("6");
		seven.addNeighbor("4");

		/* 4 edges there, in another partition */
		eight.addNeighbor("9");
		nine.addNeighbor("8");
		nine.addNeighbor("10");
		ten.addNeighbor("9");

		NetworkSystem.addNode(one);
		NetworkSystem.addNode(two);
		NetworkSystem.addNode(three);
		NetworkSystem.addNode(four);
		NetworkSystem.addNode(five);
		NetworkSystem.addNode(six);
		NetworkSystem.addNode(seven);
		NetworkSystem.addNode(eight);
		NetworkSystem.addNode(nine);
		NetworkSystem.addNode(ten);

		System.out.format("Nodes (10): %d; Edges (20): %d\n", NetworkSystem
				.getNodeCount(), NetworkSystem.getEdgeCount());

		/* Test compute */
		ParallelCompute pcomputer = new ParallelCompute(threads);
		Stats.largestpartition = 0;
		pcomputer.computeBFS();
		int diameter = pcomputer.getDiameter();
		int radius = pcomputer.getRadius();
		float avpl = pcomputer.getAveragePathLength();

		System.out
				.format(
						"Nodes (10): %d; Edges (20): %d; Diameter (3): %d; Radius (1): %d; AVPL (2.40): %3.2f; LCC (7): %d\n",
						NetworkSystem.getNodeCount(), NetworkSystem
								.getEdgeCount(), diameter, radius, avpl,
						Stats.largestpartition);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new TestCompute();
	}

}
