/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package util;

import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

public class BFS {

	private Graph graph;
	private Hashtable<String, Map<String, Integer>> distances;

	public BFS(Graph g) {
		this.graph = g;
		distances = new Hashtable<String, Map<String, Integer>>();
	}

	public int getShortestPathLength(String source, String target) {
		Map<String, Integer> entry;
		if (!distances.containsKey(source)) {
			entry = distancesFromSource(source);
			distances.put(source, entry);
		} else
			entry = distances.get(source);

		if (entry.containsKey(target))
			return entry.get(target);
		else
			return -1;
	}

	private Map<String, Integer> distancesFromSource(String source) {
		Map<String, Integer> visited = new Hashtable<String, Integer>();
		Set<String> toVisit = new HashSet<String>();
		Set<String> visiting = new HashSet<String>();
		int distance = 0;

		toVisit.add(source);

		while (!toVisit.isEmpty()) {
			visiting.clear();
			visiting.addAll(toVisit);
			toVisit.clear();
			for (String n : visiting) {
				if (!(visited.containsKey(n))) {
					visited.put(n, distance);
					toVisit.addAll(graph.getSuccessors(n));
					//Algorithm.star.progress();
				}
			}
			distance++;
		}
		return visited;
	}

}
