/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package util;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

public class WeightBFS {

	private WeightedIntGraph graph;

	public WeightBFS(WeightedIntGraph g) {
		this.graph = g;
		int gsize = g.getVertexCount();
		int[] v = new int[gsize];
		for(int i=0; i < gsize; i++) {
			Arrays.fill(v, -1);
			System.out.format("  %d:\n", i);
			elaborate(v,i);
			for (int nodeid=0; nodeid < gsize; nodeid++) {
				System.out.format("    %d: %d\n", nodeid, v[nodeid]);
			}
		}
	}
	
	private Set<Integer> getToVisitAt(Set<Integer>[] toVisitAfter, int time) {
		return toVisitAfter[time];
	}
	
	private int pushVisit(int[] distances, int currentTime, int nodeid) {
		if (distances[nodeid] < 0) {
			distances[nodeid] = currentTime;
			return 1;
		}
		return 0;
	}
	
	private void addToVisitAfter(int currentTime, Set<Integer>[] toVisitAfter, Integer node, int d) {
		Set<Integer> thelist = null;
		int temp = (int) (currentTime+d);
		if (toVisitAfter[temp] != null) {
			thelist = toVisitAfter[temp];
		} else {
			thelist = new HashSet<Integer>();
			toVisitAfter[temp] = thelist;
		}
		thelist.add(node);
	}

	@SuppressWarnings("unchecked")
	private void elaborate(int[] distances, Integer source) {
		Set<Integer>[] toVisitAfter = new HashSet[Short.MAX_VALUE];
		int currentTime = 0;
		long gsize = graph.vertexSet().size();
		int count = 0;
		
		addToVisitAfter(currentTime, toVisitAfter, source, 0);

		while(count != gsize) {
			Set<Integer> toVisit = getToVisitAt(toVisitAfter, currentTime);
			if (toVisit != null) {
				for (Integer t : toVisit) {
					count += pushVisit(distances, currentTime, t);
					Set<Integer> succ = graph.getSuccessors(t);
					for (Integer n : succ) {
						addToVisitAfter(currentTime, toVisitAfter, n, Math.max(graph.getWeight(t, n),1));
					}
				}
			}
			currentTime++;
		}
	}
	
	public static void main(String[] args) {
		WeightedIntGraph wg = importYaml(Integer.parseInt(args[0]), false, args[1]);
		new WeightBFS(wg);
	}
	
	private static WeightedIntGraph importYaml(int size, boolean directed, String path) {
		WeightedIntGraph wg = new WeightedIntGraph(size, directed);
		String line = null;
		int currentNode = -1;
		BufferedReader br = null;

		try {
			br = new BufferedReader(new FileReader(path));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		try {
			while ((line = br.readLine()) != null) {
				if (line.charAt(0) != ' ') {
					// Skip this
				} else {
					StringTokenizer st = new StringTokenizer(line, ":");
					String id = new String(st.nextToken().trim());
					int idx = Integer.parseInt(id);
					if (id.equals("datastructure"))
						break;
					if (st.countTokens() > 0) {
						// Neighbor
						Double weight = null;
						if (st.hasMoreTokens()) {
							String wgs = new String(st.nextToken().trim());
							weight = Double.parseDouble(wgs);
						} else {
							weight = 1.0;
						}
						wg.addVertex(idx);
						wg.addEdge(currentNode, idx, weight.intValue());
					} else {
						// New node
						wg.addVertex(idx);
						currentNode = idx;
					}
				}
			}
			br.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return wg;
	}

}
