/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  BlåtAnt Algorithm implementation
 *  
 *  Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
package util;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;

public class WeightedIntGraph {

	private TreeMap<Integer, Set<Integer>> vertexes;
	private int[][] weights;
	private boolean directed = false;

	public WeightedIntGraph(int size, boolean directed) {
		this.directed = directed;
		vertexes = new TreeMap<Integer, Set<Integer>>();
		weights = new int[size][size];
		for (int i=0; i < size; i++) {
			for (int j=0; j < size; j++) {
				weights[i][j] = -1;
			}
		}
	}
	
	private void addWeight(Integer from, Integer to, int weight) {
		weights[from][to]=weight;
	}

	public int getWeight(Integer from, Integer to) {
		return weights[from][to];
	}

	public void addEdge(Integer from, Integer to, int weight) {
		addVertex(from);
		addVertex(to);
		addSuccessor(from, to, weight);
		if (!directed)
			addSuccessor(to, from, weight);
	}

	private boolean addSuccessor(Integer id, Integer succ, int weight) {
		Set<Integer> adj = vertexes.get(id);
		addWeight(id, succ, weight);
		return adj.add(succ);
	}

	public void addVertex(Integer id) {
		if (!vertexes.containsKey(id))
			vertexes.put(id, new HashSet<Integer>());
	}

	public Set<Integer> getSuccessors(Integer id) {
		if (vertexes.containsKey(id))
			return vertexes.get(id);
		else
			return new HashSet<Integer>();
	}
	
	public Set<Integer> vertexSet() {
		return vertexes.keySet();
	}

	public void removeNonBidirectionalEdges() {
		for (Integer v : vertexes.keySet()) {
			List<Integer> invalid = new LinkedList<Integer>();
			Set<Integer> succ = getSuccessors(v);
			for (Integer k : succ) {
				if (!getSuccessors(k).contains(v)) {
					invalid.add(k);
				}
			}
			succ.removeAll(invalid);
		}
	}

	public boolean isDirected() {
		return directed;
	}

	public int getVertexCount() {
		return vertexes.keySet().size();
	}

}
